/*
 * Copyright 2003, 2004, 2005, 2006 PathScale, Inc.  All Rights Reserved.
 */

/*

  Copyright (C) 2000, 2001 Silicon Graphics, Inc.  All Rights Reserved.

   Path64 is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Path64 is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with Path64; see the file COPYING.  If not, write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.

   Special thanks goes to SGI for their continued support to open source

*/

//  GRA -> LRA grant implementation
/////////////////////////////////////
//  
//  Description:
//
//      Not much more than a map from BBs to vectors of REGISTER_SETs.
//
/////////////////////////////////////


//  $Revision: 1.6 $
//  $Date: 04/12/21 14:57:08-08:00 $
//  $Author: bos@eng-25.internal.keyresearch.com $
//  $Source: /home/bos/bk/kpro64-pending/be/cg/gra_mon/SCCS/s.gra_grant.cxx $

#ifdef USE_PCH
#include "cg_pch.h"
#endif // USE_PCH
#pragma hdrstop

#ifdef _KEEP_RCS_ID
static char *rcs_id = "$Source: /home/bos/bk/kpro64-pending/be/cg/gra_mon/SCCS/s.gra_grant.cxx $ $Revision: 1.6 $";
#endif

#include "tracing.h"
#include "defs.h"
#include "mempool.h"
#include "bb.h"
#include "bb_map.h"
#include "gra_grant.h"
#include "register.h"
#include "gra_bb.h"
#include "gra_trace.h"



MEM_POOL grant_pool;    // Just for grants
BB_MAP grant_map;

#ifndef Is_True_On
#define GRANT_INLINE inline
#else
#define GRANT_INLINE static
#endif

/////////////////////////////////////
typedef REGISTER_SET GRANT;
/////////////////////////////////////
//
//  Not a very impressive type really.
//
/////////////////////////////////////

/////////////////////////////////////
GRANT_INLINE void
GRANT_REGISTER_SET_Set( GRANT* grant, ISA_REGISTER_CLASS rc,  REGISTER_SET  set )
/////////////////////////////////////
//  See interface description.
/////////////////////////////////////
{
  grant[rc-ISA_REGISTER_CLASS_MIN] = set;
}

/////////////////////////////////////
GRANT_INLINE REGISTER_SET
GRANT_REGISTER_SET_Get( GRANT* grant, ISA_REGISTER_CLASS rc )
/////////////////////////////////////
//
//  Get the REGISTER_SET for <rc> from <grant>.
//
/////////////////////////////////////
{
  return grant[rc-ISA_REGISTER_CLASS_MIN];
}

/////////////////////////////////////
GRANT_INLINE void
GRANT_Union1D( GRANT* grant, ISA_REGISTER_CLASS rc, REGISTER reg )
/////////////////////////////////////
//
//  Add a register to <grant>...
//
/////////////////////////////////////
{
  GRANT_REGISTER_SET_Set(
    grant,
    rc,
    REGISTER_SET_Union1(GRANT_REGISTER_SET_Get(grant,rc),reg)
  );
}

/////////////////////////////////////
GRANT_INLINE void
GRANT_UnionD( GRANT* grant, ISA_REGISTER_CLASS rc, REGISTER_SET register_set )
/////////////////////////////////////
//
//  Add the registers in <register_set> to <grant>...
//
/////////////////////////////////////
{
  GRANT_REGISTER_SET_Set(
    grant,
    rc,
    REGISTER_SET_Union(GRANT_REGISTER_SET_Get(grant,rc),register_set)
  );
}

/////////////////////////////////////
void
GRA_GRANT_Initialize(void)
/////////////////////////////////////
//  See interface description.
/////////////////////////////////////
{
  BB *bb;
  GRANT* grant_mem;
  static BOOL grant_pool_initialized = FALSE;

  if ( ! grant_pool_initialized ) {
    grant_pool_initialized = TRUE;
    MEM_POOL_Initialize(&grant_pool,"GRA register to LRA",FALSE);
    MEM_POOL_Push(&grant_pool);
  }

  grant_mem =
    TYPE_MEM_POOL_ALLOC_N(REGISTER_SET,&grant_pool,
                                       ISA_REGISTER_CLASS_COUNT*PU_BB_Count + 2);
  memset(grant_mem, 0, sizeof(GRANT)*ISA_REGISTER_CLASS_COUNT*PU_BB_Count + 2);
  grant_map = BB_MAP_Create();

  for ( bb = REGION_First_BB; bb != NULL; bb = BB_next(bb) ) {
    BB_MAP_Set(grant_map,bb,(void*) grant_mem);
    grant_mem += ISA_REGISTER_CLASS_COUNT;
  }
}

/////////////////////////////////////
void
GRA_GRANT_Transfer( BB* from_bb, BB* to_bb )
/////////////////////////////////////
//  See interface description.
/////////////////////////////////////
{
  ISA_REGISTER_CLASS rc;
  GRANT* from_grant = (GRANT*)BB_MAP_Get(grant_map,from_bb);
  GRANT* to_grant   = (GRANT*)BB_MAP_Get(grant_map,to_bb);

  FOR_ALL_ISA_REGISTER_CLASS( rc )
    GRANT_UnionD(to_grant,rc,GRANT_REGISTER_SET_Get(from_grant,rc));
}

/////////////////////////////////////
void
GRA_GRANT_Local_Register( GRA_BB* gbb, ISA_REGISTER_CLASS rc, REGISTER reg )
/////////////////////////////////////
//  See interface description.
/////////////////////////////////////
{
  GRANT* gr = (GRANT*)BB_MAP_Get(grant_map,gbb->Bb());

  /*
     Reconsider the decision to let LRA use a particular register.

     Some registers may be needed to carry implicit values between blocks.
     We must make sure that LRA does not destroy registers that may
     contain valid information, and we will do this by keeping those
     registers out of the grant that is given to LRA.

     The only implicitly defined registers that we are concerned about
     are the ones that contain the results of a function call.  Since
     calls terminate a block, the result registers are always used in
     successor blocks.  Initially, this would always be the immediate
     successor, but hyperblock optimization may move code between blocks
     and cause the use of the implicitly defined result to be several
     blocks away from the call.

     Note: this may not be the cleanest way of preventing the problem.
     In the long run, it may not even be safe if changes are made to
     how Build_Dedicated_TN works. But it works for now!
  */
#ifndef TARG_X8664
  if (!REGISTER_SET_MemberP(REGISTER_CLASS_function_value(rc), reg) ||
      !BB_call(gbb->Bb()) &&
      !GTN_SET_MemberP(BB_live_in (gbb->Bb()),Build_Dedicated_TN(rc, reg, 8)) ||
      !GTN_SET_MemberP(BB_live_out(gbb->Bb()),Build_Dedicated_TN(rc, reg, 8)))
#endif
    {
      GRA_Trace_Grant(gbb,rc,reg);
      GRANT_Union1D(gr,rc,reg);
    }
}

/////////////////////////////////////
REGISTER_SET
GRA_GRANT_Get_Local_Registers( BB* bb, ISA_REGISTER_CLASS rc )
/////////////////////////////////////
//  See interface description.
//  Description
/////////////////////////////////////
{
  return GRANT_REGISTER_SET_Get((GRANT*) BB_MAP_Get(grant_map,bb),rc);
}

/////////////////////////////////////
void
GRA_GRANT_Finalize(void)
/////////////////////////////////////
//  See interface description.
/////////////////////////////////////
{
  MEM_POOL_Push(&grant_pool);
  BB_MAP_Delete(grant_map);
}

/////////////////////////////////////
void
GRA_GRANT_Unused_Caller_Saved(void)
/////////////////////////////////////
//  See interface description.
/////////////////////////////////////
{
  ISA_REGISTER_CLASS rc;

  GRA_Trace_Grant_Unused_Caller_Saved();
  for (BB* bb = REGION_First_BB; bb != NULL; bb = BB_next(bb) ) {
    GRA_BB* gbb = gbb_mgr.Get(bb);
    FOR_ALL_ISA_REGISTER_CLASS( rc ) {
      REGISTER_SET free_regs;
      REGISTER reg;

      free_regs = REGISTER_SET_Difference(REGISTER_CLASS_caller_saves(rc),
					  gbb->Registers_Used(rc));
      FOR_ALL_REGISTER_SET_members(free_regs, reg) {
	GRA_GRANT_Local_Register(gbb, rc, reg);
      }
    }
  }
}




