/*
 * Copyright (C) 2008 Advanced Micro Devices, Inc.  All Rights Reserved.
 */

/*
 *  Copyright (C) 2007, 2008 PathScale, LLC.  All Rights Reserved.
 */

/*
 *  Copyright (C) 2007 QLogic Corporation.  All Rights Reserved.
 */

/*
 * Copyright 2003, 2004, 2005, 2006 PathScale, Inc.  All Rights Reserved.
 */

/*

  Copyright (C) 2000, 2001 Silicon Graphics, Inc.  All Rights Reserved.

   Path64 is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Path64 is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with Path64; see the file COPYING.  If not, write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.

   Special thanks goes to SGI for their continued support to open source

*/

//
// Generate an ISA containing and group TOPS with similar operands/results 
// format.
/////////////////////////////////////////////////////////
// The instructions are listed below in the order as shown below. 
//
//   1. Real and pseudo instructions
//   2. Simulated instructions.
//
// Within each category, the instructions are arranged roughly in order
// of increasing numbers of operands.
/////////////////////////////////////
//
//  $Revision: 1.177 $
//  $Date: 05/11/11 10:44:29-08:00 $
//  $Author: tkong@hyalite.keyresearch $
//  $Source: common/targ_info/isa/x8664/SCCS/s.isa_operands.cxx $

#include "topcode.h"
#include "isa_operands_gen.h"

int main()
{
  ISA_Operands_Begin("x8664");

  /* Literals... */

  const OPERAND_VALUE_TYPE simm8 =
    ISA_Lit_Opnd_Type_Create("simm8",  8,  SIGNED,   LC_simm8);
  const OPERAND_VALUE_TYPE uimm8 =
    ISA_Lit_Opnd_Type_Create("uimm8",  8,  UNSIGNED, LC_uimm8);
  const OPERAND_VALUE_TYPE simm16 =
    ISA_Lit_Opnd_Type_Create("simm16", 16, SIGNED,   LC_simm16);
  const OPERAND_VALUE_TYPE uimm16 =
    ISA_Lit_Opnd_Type_Create("uimm16", 16, UNSIGNED, LC_uimm16);
  const OPERAND_VALUE_TYPE simm32 =
    ISA_Lit_Opnd_Type_Create("simm32", 32, SIGNED,   LC_simm32);
  const OPERAND_VALUE_TYPE uimm32 =
    ISA_Lit_Opnd_Type_Create("uimm32", 32, UNSIGNED, LC_uimm32);
  const OPERAND_VALUE_TYPE simm64 =
    ISA_Lit_Opnd_Type_Create("simm64",  64, SIGNED,  LC_simm64);
  const OPERAND_VALUE_TYPE pcrel32 =
    ISA_Lit_Opnd_Type_Create("pcrel32", 32, SIGNED, LC_pcrel32);


  /* Registers... */

  const OPERAND_VALUE_TYPE int64 =
    ISA_Reg_Opnd_Type_Create("int64", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE int32 =
    ISA_Reg_Opnd_Type_Create("int32", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     32, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE int16 =
    ISA_Reg_Opnd_Type_Create("int16", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     16, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE int8 =
    ISA_Reg_Opnd_Type_Create("int8", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     8, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE rax =
    ISA_Reg_Opnd_Type_Create("rax", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rax,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE eax =
    ISA_Reg_Opnd_Type_Create("eax", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rax,
			     32, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE ecx =
    ISA_Reg_Opnd_Type_Create("ecx", ISA_REGISTER_CLASS_integer,
                 ISA_REGISTER_SUBCLASS_rcx,
                 32, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE xmm0 =
    ISA_Reg_Opnd_Type_Create("xmm0", ISA_REGISTER_CLASS_float,
			     ISA_REGISTER_SUBCLASS_xmm0,
			     128, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE ax =
    ISA_Reg_Opnd_Type_Create("ax", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rax,
			     16, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE al =
    ISA_Reg_Opnd_Type_Create("al", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rax,
			     8, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE rdx =
    ISA_Reg_Opnd_Type_Create("rdx", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rdx,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE edx =
    ISA_Reg_Opnd_Type_Create("edx", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rdx,
			     32, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE cl =
    ISA_Reg_Opnd_Type_Create("cl", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rcx,
			     8, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE rdi =
    ISA_Reg_Opnd_Type_Create("rdi", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rdi,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE rsp =
    ISA_Reg_Opnd_Type_Create("rsp", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rsp,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE esp =
    ISA_Reg_Opnd_Type_Create("esp", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rsp,
			     32, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE rbp =
    ISA_Reg_Opnd_Type_Create("rbp", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_rbp,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE r11 =
    ISA_Reg_Opnd_Type_Create("r11", ISA_REGISTER_CLASS_integer,
			     ISA_REGISTER_SUBCLASS_r11,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE fp64 =
    ISA_Reg_Opnd_Type_Create("fp64", ISA_REGISTER_CLASS_float,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     64, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE fp128 =
    ISA_Reg_Opnd_Type_Create("fp128", ISA_REGISTER_CLASS_float,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     128, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE x87 =
    ISA_Reg_Opnd_Type_Create("x87", ISA_REGISTER_CLASS_x87,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     128, SIGNED, INVALID);
  const OPERAND_VALUE_TYPE rflags =
    ISA_Reg_Opnd_Type_Create("rflags", ISA_REGISTER_CLASS_rflags,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     64, UNSIGNED, INVALID);
  const OPERAND_VALUE_TYPE eflags =
    ISA_Reg_Opnd_Type_Create("eflags", ISA_REGISTER_CLASS_rflags,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     32, UNSIGNED, INVALID);
  const OPERAND_VALUE_TYPE x87_cw =
    ISA_Reg_Opnd_Type_Create("x87_cw", ISA_REGISTER_CLASS_x87_cw,
			     ISA_REGISTER_SUBCLASS_UNDEFINED,
			     16, UNSIGNED, INVALID);

  const OPERAND_VALUE_TYPE mmx =
    ISA_Reg_Opnd_Type_Create("mmx", ISA_REGISTER_CLASS_mmx,
                             ISA_REGISTER_SUBCLASS_UNDEFINED,
                             64, SIGNED, INVALID);

  const OPERAND_VALUE_TYPE mxcsr =
  	ISA_Reg_Opnd_Type_Create("mxcsr", ISA_REGISTER_CLASS_mxcsr,
  							 ISA_REGISTER_SUBCLASS_UNDEFINED,
  							 32, SIGNED, INVALID);
  
 
  /* Enums */

  const OPERAND_VALUE_TYPE pfhint =
    ISA_Enum_Opnd_Type_Create("pfhint", 8, UNSIGNED, EC_pfhint);

  /* Operand uses... */

  // a qualifying predicate
  const OPERAND_USE_TYPE predicate = Create_Operand_Use("predicate");
  // a base address (for memory insts)
  const OPERAND_USE_TYPE base = Create_Operand_Use("base");
  // an index added to a base
  const OPERAND_USE_TYPE index = Create_Operand_Use("index");
  // a scale to be multiplied with index 
  const OPERAND_USE_TYPE scale = Create_Operand_Use("scale");
  // an offset added to a base (implies immed val)
  const OPERAND_USE_TYPE offset = Create_Operand_Use("offset");
  // the target of a branch
  const OPERAND_USE_TYPE target = Create_Operand_Use("target");
  // value to be stored
  const OPERAND_USE_TYPE storeval = Create_Operand_Use("storeval");
  // first/left operand or result of an alu operator
  const OPERAND_USE_TYPE opnd1 = Create_Operand_Use("opnd1");
  // second/right operand of an alu operator
  const OPERAND_USE_TYPE opnd2 = Create_Operand_Use("opnd2");
  // third operand of an alu operator
  const OPERAND_USE_TYPE opnd3 = Create_Operand_Use("opnd3");
  // fourth operand of an alu operator
  const OPERAND_USE_TYPE opnd4 = Create_Operand_Use("opnd4");
  // fifth operand of an alu operator
  const OPERAND_USE_TYPE opnd5 = Create_Operand_Use("opnd5");
  // addend/subtrahend operand of a madd
  const OPERAND_USE_TYPE maddend = Create_Operand_Use("maddend");
  // operand has same register as result
  const OPERAND_USE_TYPE same_res = Create_Operand_Use("same_res");

  Instruction_Group("no operand",
		    TOP_nop,
		    TOP_noop,
		    TOP_enter,
		    TOP_ret,
		    TOP_asm,
		    TOP_begin_pregtn,
		    TOP_end_pregtn,
		    TOP_fwd_bar,
		    TOP_bwd_bar,
		    TOP_mfence,
		    TOP_lfence,
		    TOP_sfence,
		    TOP_emms,
                    TOP_swp_start,
                    TOP_swp_stop,
		    TOP_UNDEFINED);
	Instruction_Group("sse2 clflush",
	                  TOP_clflush,
		                TOP_UNDEFINED);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("ret imm16",
		    TOP_reti,
		    TOP_UNDEFINED);
  Operand( 0, simm16, opnd1 );

  Instruction_Group("int load imm32",
		    TOP_ldc32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, simm32, opnd1);

  Instruction_Group("int load imm64",
		    TOP_ldc64,
		    TOP_movabsq,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, simm64, opnd1);

  Instruction_Group( "int8 arithmetic",
		     TOP_and8,
		     TOP_or8,
		     TOP_xor8,
		     TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8, opnd1);
  Operand(1, int8, opnd2);


  Instruction_Group( "int16 arithmetic",
		     TOP_and16,
		     TOP_or16,
		     TOP_xor16,
		     TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);
  Operand(1, int16, opnd2);

  Instruction_Group( "int32 arithmetic",
		     TOP_add32,
		     TOP_adc32,
		     TOP_and32,
		     TOP_or32,
		     TOP_sub32,
		     TOP_sbb32,
		     TOP_xor32,
		     TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int32, opnd2);

  Instruction_Group( "int64 arithmetic",
		     TOP_add64,
		     TOP_and64,
		     TOP_or64,
		     TOP_sub64,
		     TOP_xor64,
		     TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, int64, opnd2);

  Instruction_Group( "vector arithmetic",
  		     TOP_mul128v16,
  		     TOP_mulhw128v16,
		     TOP_pmuludq128,
				 TOP_pavgb128,
				 TOP_pavgw128,
				 TOP_psadbw128,
		     TOP_add128v8,
		     TOP_add128v16,
		     TOP_add128v32,
		     TOP_add128v64,
				 TOP_paddusb128,
				 TOP_paddusw128,
             TOP_paddsb128,
             TOP_paddsw128,
		     TOP_fadd128v32,
		     TOP_fadd128v64,
		     TOP_fhadd128v32,
		     TOP_fhadd128v64,
		     TOP_faddsub128v32,
		     TOP_faddsub128v64,
				 TOP_pmulhuw128,
		     TOP_sub128v8,
		     TOP_sub128v16,
		     TOP_sub128v32,
		     TOP_sub128v64,
		     TOP_psubsb128,
		     TOP_psubsw128,
		     TOP_fsub128v32,
		     TOP_fsub128v64,
		     TOP_fhsub128v32,
		     TOP_fhsub128v64,
		     TOP_and128v8,
		     TOP_and128v16,
		     TOP_and128v32,
		     TOP_and128v64,
		     TOP_andn128v8,
		     TOP_andn128v16,
		     TOP_andn128v32,
		     TOP_andn128v64,
		     TOP_fand128v32,
		     TOP_fand128v64,
		     TOP_fandn128v64,
		     TOP_or128v8,
		     TOP_or128v16,
		     TOP_or128v32,
		     TOP_or128v64,
		     TOP_for128v32,
		     TOP_for128v64,
		     TOP_xor128v8,
		     TOP_xor128v16,
		     TOP_xor128v32,
		     TOP_xor128v64,
		     TOP_fxor128v32,
		     TOP_fxor128v64,
		     TOP_xorps,
		     TOP_xorpd,
		     TOP_orps,
		     TOP_orpd,
		     TOP_andps,
		     TOP_andpd,
		     TOP_fmax128v32,
		     TOP_fmax128v64,
		     TOP_fmin128v32,
		     TOP_fmin128v64,
		     TOP_fdiv128v32,
		     TOP_fdiv128v64,
		     TOP_fmul128v32,
		     TOP_fmul128v64,
		     TOP_subus128v16,
		     TOP_cmpgt128v8,
		     TOP_cmpgt128v16,
		     TOP_cmpgt128v32,
		     TOP_cmpeq128v8,
		     TOP_cmpeq128v16,
		     TOP_cmpeq128v32,
				 TOP_pcmpeqq,
		     TOP_max128v8,
		     TOP_max128v16,
		     TOP_min128v8,
		     TOP_min128v16,
             TOP_pand128,
             TOP_pandn128,
						 TOP_por128,
		     TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);

  Instruction_Group( "vector arithmetic mem opnd",
			   TOP_addx128v8,
			   TOP_addx128v16,
			   TOP_addx128v32,
			   TOP_addx128v64,
			   TOP_faddx128v32,
			   TOP_faddx128v64,
			   TOP_fhaddx128v32,
			   TOP_fhaddx128v64,
			   TOP_faddsubx128v32,
			   TOP_faddsubx128v64,
			   TOP_andx128v8,
			   TOP_andx128v16,
			   TOP_andx128v32,
			   TOP_andx128v64,
			   TOP_andnx128v8,
			   TOP_andnx128v16,
			   TOP_andnx128v32,
			   TOP_andnx128v64,
			   TOP_fandx128v32,
			   TOP_fandx128v64,
			   TOP_fandnx128v64,
			   TOP_orx128v8,
			   TOP_orx128v16,
			   TOP_orx128v32,
			   TOP_orx128v64,
			   TOP_forx128v32,
			   TOP_forx128v64,
			   TOP_xorx128v8,
			   TOP_xorx128v16,
			   TOP_xorx128v32,
			   TOP_xorx128v64,
			   TOP_fxorx128v32,
			   TOP_fxorx128v64,
			   TOP_fmaxx128v32,
			   TOP_fmaxx128v64,
			   TOP_fminx128v32,
			   TOP_fminx128v64,
			   TOP_fdivx128v32,
			   TOP_fdivx128v64,
			   TOP_fmulx128v32,
			   TOP_fmulx128v64,
			   TOP_subx128v8,
			   TOP_subx128v16,
			   TOP_subx128v32,
			   TOP_subx128v64,
			   TOP_fsubx128v32,
			   TOP_fsubx128v64,
			   TOP_fhsubx128v32,
		           TOP_fhsubx128v64,
		           TOP_cmpgtx128v8,
		           TOP_cmpgtx128v16,
		           TOP_cmpgtx128v32,
		           TOP_cmpeqx128v8,
		           TOP_cmpeqx128v16,
		           TOP_cmpeqx128v32,
		           TOP_maxx128v8,
		           TOP_maxx128v16,
		           TOP_minx128v8,
		           TOP_minx128v16,
			   TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  
  Instruction_Group( "vector arithmetic mem opnd w/ scaled index",
			   TOP_addxx128v8,
			   TOP_addxx128v16,
			   TOP_addxx128v32,
			   TOP_addxx128v64,
			   TOP_faddxx128v32,
			   TOP_faddxx128v64,
			   TOP_fhaddxx128v32,
			   TOP_fhaddxx128v64,
			   TOP_faddsubxx128v32,
			   TOP_faddsubxx128v64,
			   TOP_andxx128v8,
			   TOP_andxx128v16,
			   TOP_andxx128v32,
			   TOP_andxx128v64,
			   TOP_andnxx128v8,
			   TOP_andnxx128v16,
			   TOP_andnxx128v32,
			   TOP_andnxx128v64,
			   TOP_fandxx128v32,
			   TOP_fandxx128v64,
			   TOP_fandnxx128v64,
			   TOP_orxx128v8,
			   TOP_orxx128v16,
			   TOP_orxx128v32,
			   TOP_orxx128v64,
			   TOP_forxx128v32,
			   TOP_forxx128v64,
			   TOP_xorxx128v8,
			   TOP_xorxx128v16,
			   TOP_xorxx128v32,
			   TOP_xorxx128v64,
			   TOP_fxorxx128v32,
			   TOP_fxorxx128v64,
			   TOP_fmaxxx128v32,
			   TOP_fmaxxx128v64,
			   TOP_fminxx128v32,
			   TOP_fminxx128v64,
			   TOP_fdivxx128v32,
			   TOP_fdivxx128v64,
			   TOP_fmulxx128v32,
			   TOP_fmulxx128v64,
			   TOP_subxx128v8,
			   TOP_subxx128v16,
			   TOP_subxx128v32,
			   TOP_subxx128v64,
			   TOP_fsubxx128v32,
			   TOP_fsubxx128v64,
			   TOP_fhsubxx128v32,
			   TOP_fhsubxx128v64,
		           TOP_cmpgtxx128v8,
		           TOP_cmpgtxx128v16,
		           TOP_cmpgtxx128v32,
		           TOP_cmpeqxx128v8,
		           TOP_cmpeqxx128v16,
		           TOP_cmpeqxx128v32,
		           TOP_maxxx128v8,
		           TOP_maxxx128v16,
		           TOP_minxx128v8,
		           TOP_minxx128v16,
			   TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);
  
  Instruction_Group( "vector arithmetic mem opnd w/ scaled index w/o base",
			   TOP_addxxx128v8,
			   TOP_addxxx128v16,
			   TOP_addxxx128v32,
			   TOP_addxxx128v64,
			   TOP_faddxxx128v32,
			   TOP_faddxxx128v64,
			   TOP_fhaddxxx128v32,
			   TOP_fhaddxxx128v64,
			   TOP_faddsubxxx128v32,
			   TOP_faddsubxxx128v64,
			   TOP_andxxx128v8,
			   TOP_andxxx128v16,
			   TOP_andxxx128v32,
			   TOP_andxxx128v64,
			   TOP_andnxxx128v8,
			   TOP_andnxxx128v16,
			   TOP_andnxxx128v32,
			   TOP_andnxxx128v64,
			   TOP_fandxxx128v32,
			   TOP_fandxxx128v64,
			   TOP_fandnxxx128v64,
			   TOP_orxxx128v8,
			   TOP_orxxx128v16,
			   TOP_orxxx128v32,
			   TOP_orxxx128v64,
			   TOP_forxxx128v32,
			   TOP_forxxx128v64,
			   TOP_xorxxx128v8,
			   TOP_xorxxx128v16,
			   TOP_xorxxx128v32,
			   TOP_xorxxx128v64,
			   TOP_fxorxxx128v32,
			   TOP_fxorxxx128v64,
			   TOP_fmaxxxx128v32,
			   TOP_fmaxxxx128v64,
			   TOP_fminxxx128v32,
			   TOP_fminxxx128v64,
			   TOP_fdivxxx128v32,
			   TOP_fdivxxx128v64,
			   TOP_fmulxxx128v32,
			   TOP_fmulxxx128v64,
			   TOP_subxxx128v8,
			   TOP_subxxx128v16,
			   TOP_subxxx128v32,
			   TOP_subxxx128v64,
			   TOP_fsubxxx128v32,
			   TOP_fsubxxx128v64,
			   TOP_fhsubxxx128v32,
			   TOP_fhsubxxx128v64,
		           TOP_cmpgtxxx128v8,
		           TOP_cmpgtxxx128v16,
		           TOP_cmpgtxxx128v32,
		           TOP_cmpeqxxx128v8,
		           TOP_cmpeqxxx128v16,
		           TOP_cmpeqxxx128v32,
		           TOP_maxxxx128v8,
		           TOP_maxxxx128v16,
		           TOP_minxxx128v8,
		           TOP_minxxx128v16,
			   TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group( "int8 arithmetic mem opnd",
		     TOP_xorx8,
		     TOP_orx8,
		     TOP_andx8,
		     TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "int16 arithmetic mem opnd",
		     TOP_xorx16,
		     TOP_orx16,
		     TOP_andx16,
		     TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "int32 arithmetic mem opnd",
		     TOP_addx32,
		     TOP_subx32,
		     TOP_xorx32,
		     TOP_orx32,
		     TOP_andx32,
		     TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "int64 arithmetic mem opnd",
		     TOP_addx64,
		     TOP_subx64,
		     TOP_xorx64,
		     TOP_orx64,
		     TOP_andx64,
		     TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "int8 arithmetic mem opnd w/ scaled index",
		     TOP_xorxx8,
		     TOP_orxx8,
		     TOP_andxx8,
		     TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8,  opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group( "int16 arithmetic mem opnd w/ scaled index",
		     TOP_xorxx16,
		     TOP_orxx16,
		     TOP_andxx16,
		     TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group( "int32 arithmetic mem opnd w/ scaled index",
		     TOP_addxx32,
		     TOP_subxx32,
		     TOP_xorxx32,
		     TOP_orxx32,
		     TOP_andxx32,
		     TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group( "int64 arithmetic mem opnd w/ scaled index",
		     TOP_addxx64,
		     TOP_subxx64,
		     TOP_xorxx64,
		     TOP_orxx64,
		     TOP_andxx64,
		     TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group( "int8 arithmetic mem opnd w/ scaled index w/o base",
		     TOP_xorxxx8,
		     TOP_orxxx8,
		     TOP_andxxx8,
		     TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8,  opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group( "int16 arithmetic mem opnd w/ scaled index w/o base",
		     TOP_xorxxx16,
		     TOP_orxxx16,
		     TOP_andxxx16,
		     TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group( "int32 arithmetic mem opnd w/ scaled index w/o base",
		     TOP_addxxx32,
		     TOP_subxxx32,
		     TOP_xorxxx32,
		     TOP_orxxx32,
		     TOP_andxxx32,
		     TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group( "int64 arithmetic mem opnd w/ scaled index w/o base",
		     TOP_addxxx64,
		     TOP_subxxx64,
		     TOP_xorxxx64,
		     TOP_orxxx64,
		     TOP_andxxx64,
		     TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group( "unary int8 arithmetic",
  		     TOP_inc8,
		     TOP_dec8,
		     TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8, opnd1);

  Instruction_Group( "unary int16 arithmetic",
  		     TOP_inc16,
  		     TOP_dec16,
		     TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);

  Instruction_Group( "unary int32 arithmetic",
		     TOP_neg32,
		     TOP_not32,
  		     TOP_inc32,
		     TOP_dec32,
		     TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);

  Instruction_Group( "unary int64 arithmetic",
		     TOP_neg64,
		     TOP_not64,
		     TOP_inc64,
		     TOP_dec64,
		     TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);

  Instruction_Group("int32 arithmetic with simm",
		    TOP_xori32,
		    TOP_ori32,
		    TOP_andi32,
		    TOP_addi32,
		    TOP_adci32,
		    TOP_subi32,
		    TOP_sbbi32,
		    TOP_imuli32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, simm32, opnd2);
  
  Instruction_Group("int64 arithmetic with simm",
		    TOP_xori64,
		    TOP_ori64,
		    TOP_andi64,
		    TOP_addi64,
		    TOP_subi64,
		    TOP_imuli64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, simm32, opnd2);

  Instruction_Group("int8 load w/o base or index (movabs)",
		    TOP_ld8_abs,
		    TOP_UNDEFINED);
  Result(0, al);
  Operand(0, simm64, offset);

  Instruction_Group("int16 load w/o base or index (movabs)",
		    TOP_ld16_abs,
		    TOP_UNDEFINED);
  Result(0, ax);
  Operand(0, simm64, offset);

  Instruction_Group("int32 load w/o base or index (movabs)",
		    TOP_ld32_abs,
		    TOP_UNDEFINED);
  Result(0, eax);
  Operand(0, simm64, offset);

  Instruction_Group("int32 load w/o base or index",
		    TOP_ld8_32_n32,
		    TOP_ldu8_32_n32,
		    TOP_ld16_32_n32,
		    TOP_ldu16_32_n32,
		    TOP_ld32_n32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, simm32, offset);

  Instruction_Group("int64 load w/o base or index",
		    TOP_ld64_off,
		    TOP_ld8_64_off,
		    TOP_ldu8_64_off,
		    TOP_ld16_64_off,
		    TOP_ldu16_64_off,
		    TOP_ld32_64_off,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, simm32, offset);

  Instruction_Group("int64 load w/o base or index (movabs)",
		    TOP_ld64_abs,
		    TOP_UNDEFINED);
  Result(0, rax);
  Operand(0, simm64, offset);

  Instruction_Group("int32 load",
		    TOP_ld8_32,
		    TOP_ldu8_32,
		    TOP_ld16_32,
		    TOP_ldu16_32,
		    TOP_ld32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("int64 load",
		    TOP_ld8_64,
		    TOP_ldu8_64,
		    TOP_ld16_64,
		    TOP_ldu16_64,
		    TOP_ld32_64,
		    TOP_ld64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("int32 load from %gs segment w/o base or index",
		    TOP_ld32_gs_seg_off,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, simm32, offset);

  Instruction_Group("int64 load from %fs segment w/o base or index",
		    TOP_ld64_fs_seg_off,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, simm32, offset);

  Instruction_Group("int8 store w/o base or index (movabs)",
		    TOP_store8_abs,
		    TOP_UNDEFINED);
  Operand(0, al, storeval);
  Operand(1, simm64, offset);

  Instruction_Group("int8 store w/o base or index",
		    TOP_store8_n32,
		    TOP_UNDEFINED);
  Operand(0, int8, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("int16 store w/o base or index (movabs)",
		    TOP_store16_abs,
		    TOP_UNDEFINED);
  Operand(0, ax, storeval);
  Operand(1, simm64, offset);

  Instruction_Group("int16 store w/o base or index",
		    TOP_store16_n32,
		    TOP_UNDEFINED);
  Operand(0, int16, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("int32 store w/o base or index (movabs)",
		    TOP_store32_abs,
		    TOP_UNDEFINED);
  Operand(0, eax, storeval);
  Operand(1, simm64, offset);

  Instruction_Group("int32 store w/o base or index",
		    TOP_store32_n32,
		    TOP_UNDEFINED);
  Operand(0, int32, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("int64 store w/o base or index",
		    TOP_store64_off,
		    TOP_UNDEFINED);
  Operand(0, int64, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("int64 store w/o base or index (movabs)",
		    TOP_store64_abs,
		    TOP_UNDEFINED);
  Operand(0, rax, storeval);
  Operand(1, simm64, offset);

  Instruction_Group("int8 store",
		    TOP_store8,
		    TOP_UNDEFINED);
  Operand(0, int8, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int16 store",
		    TOP_store16,
		    TOP_UNDEFINED);
  Operand(0, int16, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int32 store",
		    TOP_store32,
		    TOP_storenti32,
		    TOP_UNDEFINED);
  Operand(0, int32, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int64 store",
		    TOP_store64,
		    TOP_storenti64,
		    TOP_UNDEFINED);
  Operand(0, int64, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("prefetch",
		    TOP_prefetch,
		    TOP_prefetchw,
		    TOP_prefetcht0,
		    TOP_prefetcht1,
		    TOP_prefetchnta,
		    TOP_UNDEFINED);
  Operand(0, pfhint);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("prefetchx",
		    TOP_prefetchx,
		    TOP_prefetchwx,
		    TOP_prefetcht0x,
		    TOP_prefetcht1x,
		    TOP_prefetchntax,
		    TOP_UNDEFINED);
  Operand(0, pfhint);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  Operand(3, int64, index);
  Operand(4, uimm8, scale);

  Instruction_Group("prefetchxx",
		    TOP_prefetchxx,
		    TOP_prefetchwxx,
		    TOP_prefetcht0xx,
		    TOP_prefetcht1xx,
		    TOP_prefetchntaxx,
		    TOP_UNDEFINED);
  Operand(0, pfhint);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("int8_32 mov",
		    TOP_movsbl,
		    TOP_movzbl,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int8, opnd1);

  Instruction_Group("int16_32 mov",
		    TOP_movswl,
		    TOP_movzwl,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int16, opnd1);

  Instruction_Group("int8_64 mov",
		    TOP_movsbq,
		    TOP_movzbq,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int8, opnd1);

  Instruction_Group("int16_64 mov",
		    TOP_movswq,
		    TOP_movzwq,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int16, opnd1);

  Instruction_Group("int32_64 sign-ext mov",
		    TOP_movslq,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int32, opnd1);

  // Print the result as a 32-bit register even though the result is 64-bit,
  // since CG emits mov32 for movzlq.  mov32 expects a 32-bit result.
  Instruction_Group("int32_64 zero-ext mov",
		    TOP_movzlq,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);

  Instruction_Group("int32 mov",
		    TOP_mov32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);

  Instruction_Group("int64 mov",
		    TOP_mov64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);

  Instruction_Group("int32 mult",
		    TOP_imul32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int32, opnd2);

  Instruction_Group("int64 mult",
		    TOP_imul64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, int64, opnd2);

  Instruction_Group("int32 mult uses eax and edx registers",
		    TOP_mul32,
		    TOP_imulx32,
		    TOP_UNDEFINED);
  Result(0, eax);
  Result(1, edx);
  Operand(0, eax, opnd1);
  Operand(1, int32, opnd2);

  Instruction_Group("int64 mult uses rax and rdx registers",
		    TOP_imulx64,
		    TOP_mulx64,
		    TOP_UNDEFINED);
  Result(0, rax);
  Result(1, rdx);
  Operand(0, rax, opnd1);
  Operand(1, int64, opnd2);

  Instruction_Group("div32",
		    TOP_div32,
		    TOP_UNDEFINED);
  Result(0, eax);
  Result(1, edx);
  Operand(0, eax, opnd1);
  Operand(1, edx, opnd2);
  Operand(2, int32, opnd3);

  Instruction_Group("div64",
		    TOP_div64,
		    TOP_UNDEFINED);
  Result(0, rax);
  Result(1, rdx);
  Operand(0, rax, opnd1);
  Operand(1, rdx, opnd2);
  Operand(2, int64, opnd3);

  Instruction_Group("idiv32",
		    TOP_idiv32,
		    TOP_UNDEFINED);
  Result(0, eax);
  Result(1, edx);
  Operand(0, eax, opnd1);
  Operand(1, edx, opnd2);
  Operand(2, int32, opnd3);

  Instruction_Group("idiv64",
		    TOP_idiv64,
		    TOP_UNDEFINED);
  Result(0, rax);
  Result(1, rdx);
  Operand(0, rax, opnd1);
  Operand(1, rdx, opnd2);
  Operand(2, int64, opnd3);

  Instruction_Group("int8 set",
		    TOP_setb,
		    TOP_setae,
		    TOP_setp,
		    TOP_setnp,
		    TOP_sete,
		    TOP_setne,
		    TOP_setbe,
		    TOP_seta,
		    TOP_setl,
		    TOP_setge,
		    TOP_setle,
		    TOP_setg,
		    TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, rflags, opnd1);

  Instruction_Group("cond int mov",
		    TOP_cmovb,
		    TOP_cmovae,
		    TOP_cmovp,
		    TOP_cmovnp,
		    TOP_cmove,
		    TOP_cmovne,
		    TOP_cmovbe,
		    TOP_cmova,
		    TOP_cmovl,
		    TOP_cmovge,
		    TOP_cmovle,
		    TOP_cmovg,
		    TOP_cmovs,
		    TOP_cmovns,
		    TOP_cmovo,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64,  opnd1);
  Operand(1, rflags, opnd2);

  Instruction_Group("cond x87 mov",
		    TOP_fcmovb,
		    TOP_fcmovbe,
		    TOP_fcmovnb,
		    TOP_fcmovnbe,
		    TOP_fcmove,
		    TOP_fcmovne,
		    TOP_fcmovu,
		    TOP_fcmovnu,
		    TOP_UNDEFINED);
  Result(0, x87);
  Operand(0, x87,  opnd1);
  Operand(1, rflags, opnd2);

  Instruction_Group("fp 2 fp",
		    TOP_movsd,
		    TOP_movss,
		    TOP_cvtss2sd,
		    TOP_cvtsd2ss,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64, opnd1);

  Instruction_Group("vector cvt",
		    TOP_cvtdq2pd,
		    TOP_cvtdq2ps,
		    TOP_cvtps2pd,
		    TOP_cvtpd2ps,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);

  Instruction_Group("vector cvt w/ trunc",
		    TOP_cvtps2dq,
		    TOP_cvttps2dq,
		    TOP_cvtpd2dq,
		    TOP_cvttpd2dq,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);

  Instruction_Group("vector cvt I",
		    TOP_cvtdq2pd_x,
		    TOP_cvtdq2ps_x,
		    TOP_cvtps2pd_x,
		    TOP_cvtpd2ps_x,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("vector cvt w/ trunc I",
		    TOP_cvtps2dq_x,
		    TOP_cvttps2dq_x,
		    TOP_cvttpd2dq_x,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("vector cvt II",
		    TOP_cvtdq2pd_xx,
		    TOP_cvtdq2ps_xx,
		    TOP_cvtps2pd_xx,
		    TOP_cvtpd2ps_xx,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset); 

  Instruction_Group("vector cvt w/ trunc II",
		    TOP_cvtps2dq_xx,
		    TOP_cvttps2dq_xx,
		    TOP_cvttpd2dq_xx,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset); 

  Instruction_Group("vector cvt III",
		    TOP_cvtdq2pd_xxx,
		    TOP_cvtdq2ps_xxx,
		    TOP_cvtps2pd_xxx,
		    TOP_cvtpd2ps_xxx,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("vector cvt w/ trunc III",
		    TOP_cvtps2dq_xxx,
		    TOP_cvttps2dq_xxx,
		    TOP_cvttpd2dq_xxx,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("int32 2 float",
  		    TOP_movg2x,
		    TOP_cvtsi2sd,
		    TOP_cvtsi2ss,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, int32, opnd1);

  Instruction_Group("int64 2 float",
	  	    TOP_movg2x64,
		    TOP_cvtsi2sdq,
		    TOP_cvtsi2ssq,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, int64, opnd1);

  Instruction_Group("fp 2 int32",
	  	    TOP_movx2g,
		    TOP_cvtss2si,
		    TOP_cvtsd2si,
		    TOP_cvttss2si,
		    TOP_cvttsd2si,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, fp64, opnd1);

  Instruction_Group("fp 2 int64",
		    TOP_movx2g64,
		    TOP_cvtss2siq,
		    TOP_cvtsd2siq,
		    TOP_cvttss2siq,
		    TOP_cvttsd2siq,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, fp64, opnd1);

	Instruction_Group("mov fp 2 128",
				TOP_pmovsxbw,
				TOP_pmovsxbd,
				TOP_pmovsxbq,
				TOP_pmovsxwd,
				TOP_pmovsxwq,
				TOP_pmovsxdq,
				TOP_pmovzxbw,
				TOP_pmovzxbd,
				TOP_pmovzxbq,
				TOP_pmovzxwd,
				TOP_pmovzxwq,
				TOP_pmovzxdq,
				TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);


  Instruction_Group("shifts8",
		    TOP_rori8,
		    TOP_roli8,
		    TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8, opnd1);
  Operand(1, uimm8, opnd2);

  Instruction_Group("shifts16",
		    TOP_rori16,
		    TOP_roli16,
		    TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);
  Operand(1, uimm8, opnd2);

  Instruction_Group("shifts32",
		    TOP_sari32,
		    TOP_shli32,
		    TOP_shri32,
		    TOP_rori32,
		    TOP_roli32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, uimm8, opnd2);

  Instruction_Group("shifts64",
		    TOP_sari64,
		    TOP_shli64,
		    TOP_shri64,
		    TOP_rori64,
		    TOP_roli64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, uimm8, opnd2);

  Instruction_Group("variable shifts32",
		    TOP_sar32,
		    TOP_shl32,
		    TOP_shr32,
		    TOP_ror8,
		    TOP_ror16,
		    TOP_ror32,
		    TOP_rol8,
		    TOP_rol16,
		    TOP_rol32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, cl,    opnd2);

  Instruction_Group("variable shift left double",
		    TOP_shld32,
		    TOP_shrd32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int32, opnd2);
  Operand(2, cl,    opnd3);

  Instruction_Group("shift left double",
		    TOP_shldi32,
		    TOP_shrdi32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int32, opnd2);
  Operand(2, uimm8, opnd3);

  Instruction_Group("variable shifts64",
		    TOP_sar64,
		    TOP_shl64,
		    TOP_shr64,
		    TOP_ror64,
		    TOP_rol64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, cl,    opnd2);

  Instruction_Group("sign32 extend",
		    TOP_cltd,
		    TOP_UNDEFINED);
  Result(0, eax);
  Result(1, edx);
  Operand(0, eax, opnd1);


  Instruction_Group("sign64 extend",
		    TOP_cqto,
		    TOP_UNDEFINED);
  Result(0, rax);
  Result(1, rdx);
  Operand(0, rax, opnd1);

  Instruction_Group("float load w/o base or index",
		    TOP_ldsd_n32,
		    TOP_ldss_n32,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, simm32, offset);

  Instruction_Group("float load",
		    TOP_ldsd,
		    TOP_ldss,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("float convert",
		    TOP_cvtsd2ss_x,
		    TOP_cvtsi2sd_x,
		    TOP_cvtsi2ss_x,
		    TOP_cvtsi2sdq_x,
		    TOP_cvtsi2ssq_x,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("x87-stack float load",
		    TOP_fld,
		    TOP_UNDEFINED);
  Operand(0, x87);

  Instruction_Group("x87 float load",
		    TOP_flds,
		    TOP_fldl,
		    TOP_fldt,
		    TOP_filds,
		    TOP_fildl,
		    TOP_fildll,
		    TOP_UNDEFINED);
  Result(0, x87);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("x87 float load w/o index or base",
		    TOP_flds_n32,
		    TOP_fldl_n32,
		    TOP_fldt_n32,
		    TOP_UNDEFINED);
  Result(0, x87);
  Operand(0, simm32, offset);

  Instruction_Group("x87 control-word load",
		    TOP_fldcw,
		    TOP_UNDEFINED);
  Result(0, x87_cw);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("x87 control-word store",
		    TOP_fnstcw,
		    TOP_UNDEFINED);
  Operand(0, x87_cw, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("float load vector w/o base or index",
		    TOP_lddqa_n32,
		    TOP_lddqu_n32,
		    TOP_ldapd_n32,
		    TOP_ldupd_n32,
		    TOP_ldaps_n32,
		    TOP_ldups_n32,
		    TOP_ldlps_n32,
		    TOP_ldhps_n32,
		    TOP_ldlpd_n32,
		    TOP_ldhpd_n32,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, simm32, offset);

  Instruction_Group("float load vector",
		    TOP_lddqa,
		    TOP_lddqu,
		    TOP_ldhps,
		    TOP_ldlps,
		    TOP_ldhpd,
		    TOP_ldlpd,
		    TOP_ldapd,
		    TOP_ldaps,
		    TOP_ldupd,
		    TOP_ldups,
		    TOP_fmovsldupx,
		    TOP_fmovshdupx,
		    TOP_fmovddupx,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

	Instruction_Group("sse4.1 load MOVNTDQA",
				TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, rax, base);
	Operand(1, simm32, offset);

  Instruction_Group("float store w/o base or index",
		    TOP_stsd_n32,
		    TOP_stss_n32,
		    TOP_UNDEFINED);
  Operand(0, fp64, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("float store",
		    TOP_stsd,
		    TOP_stntsd,
		    TOP_stss,
		    TOP_stntss,
		    TOP_UNDEFINED);
  Operand(0, fp64, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("float store vector w/o base or index",
		    TOP_stdqa_n32,
		    TOP_stdqu_n32,
		    TOP_stapd_n32,
		    TOP_staps_n32,
		    TOP_stlps_n32,
		    TOP_sthps_n32,
		    TOP_stlpd_n32,
		    TOP_sthpd_n32,
		    TOP_UNDEFINED);
  Operand(0, fp128, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("float store vector",
		    TOP_stdqa,
		    TOP_stdqu,
		    TOP_sthpd,
		    TOP_stlpd,
		    TOP_sthps,
		    TOP_stlps,
		    TOP_stapd,
		    TOP_staps,
		    TOP_stupd,
		    TOP_stups,
		    TOP_stntpd,
		    TOP_stntps,
		    TOP_storenti128,
		    TOP_storelpd,
		    TOP_UNDEFINED);
  Operand(0, fp128, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("x87 float store",
		    TOP_fistps,
		    TOP_fistpl,
		    TOP_fistpll,
		    TOP_fists,
		    TOP_fistl,
		    TOP_fisttps,
		    TOP_fisttpl,
		    TOP_fisttpll,
		    TOP_UNDEFINED);
  Operand(0, x87, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("x87 store and pop",
		    TOP_fst,
		    TOP_fstp,
		    TOP_UNDEFINED);
  Operand(0, x87, opnd1);

  Instruction_Group("x87 float store",
		    TOP_fstps,
		    TOP_fstpl,
		    TOP_fstpt,
		    TOP_fsts,
		    TOP_fstl,
		    TOP_UNDEFINED);
  Operand(0, x87, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("x87 float store w/o base or index",
		    TOP_fstps_n32,
		    TOP_fstpl_n32,
		    TOP_fstpt_n32,
		    TOP_fsts_n32,
		    TOP_fstl_n32,
		    TOP_UNDEFINED);
  Operand(0, x87, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("unpack",
		    TOP_unpckhps,
		    TOP_unpcklps,
		    TOP_unpckhpd,
		    TOP_unpcklpd,
		    TOP_punpcklbw,
				TOP_packsswb128,
				TOP_packssdw128,
				TOP_packuswb128,
				TOP_punpckhbw128,
				TOP_punpckhwd128,
				TOP_punpckhdq128,
				TOP_punpckhqdq,
				TOP_punpcklbw128,
				TOP_punpcklwd128,
				TOP_punpckldq128,
				TOP_punpcklqdq,
		    TOP_punpcklwd,
		    TOP_punpckldq,
				TOP_packusdw,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);

  Instruction_Group("shuffle",
		    TOP_shufps,
		    TOP_shufpd,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);
  Operand(2, uimm8, opnd3);

  Instruction_Group("shuffle-int",
		    TOP_pshufd,
		    TOP_pshufw,
		    TOP_pshuflw,
		    TOP_pshufhw,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, uimm8, opnd3);

  Instruction_Group("move-high-low",
		    TOP_movlhps,
		    TOP_movhlps,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);

  Instruction_Group("packed shifts w/ immediate",
		    TOP_psrldq,
		    TOP_psrlq128v64, // psrlq with immediate
		    TOP_pslldq,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, simm8, opnd2);

  Instruction_Group("packed shifts",
		    TOP_psllw,
		    TOP_pslld,
		    TOP_psllq,
		    TOP_psrlw,
		    TOP_psrld,
		    TOP_psrlq,
		    TOP_psraw,
		    TOP_psrad,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);

  Instruction_Group("float load indexed",
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, int64, base);
  Operand(1, int64);

  Instruction_Group("float store indexed",
		    TOP_UNDEFINED);
  Operand(0, fp64, storeval);
  Operand(1, int64, base);
  Operand(2, int64);

  Instruction_Group("float arithmetic",
		    TOP_andnps,
		    TOP_andnpd,
		    TOP_addsd,
		    TOP_addss,
		    TOP_divsd,
		    TOP_divss,
		    TOP_subsd,
		    TOP_subss,
		    TOP_maxsd,
		    TOP_maxss,
		    TOP_minsd,
		    TOP_minss,
		    TOP_mulsd,
		    TOP_mulss,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64, opnd1);
  Operand(1, fp64, opnd2);

  Instruction_Group("x87 float exchange",
		    TOP_fxch,
		    TOP_UNDEFINED);
  Operand(0, x87, opnd1);

  Instruction_Group("x87 float arithmetic",
		    TOP_fadd,
		    TOP_faddp,
		    TOP_fsub,
		    TOP_fsubp,
		    TOP_fsubr,
		    TOP_fsubrp,
		    TOP_fmul,
		    TOP_fmulp,
		    TOP_fdiv,
		    TOP_fdivp,
		    TOP_fdivr,
		    TOP_fdivrp,
		    TOP_UNDEFINED);
  Result( 0, x87 );
  Operand(0, x87, opnd1);
  Operand(1, x87, opnd2);

  Instruction_Group("float arithmetic mem operand",
		    TOP_divxss,
		    TOP_divxsd,
		    TOP_addxss,
		    TOP_addxsd,
		    TOP_subxss,
		    TOP_subxsd,
		    TOP_mulxss,
		    TOP_mulxsd,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("float arithmetic mem operand w/ scaled-index",
		    TOP_divxxss,
		    TOP_divxxsd,
		    TOP_addxxss,
		    TOP_addxxsd,
		    TOP_subxxss,
		    TOP_subxxsd,
		    TOP_mulxxsd,
		    TOP_mulxxss,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group("float arithmetic mem operand w/ scaled-index w/o base",
		    TOP_divxxxss,
		    TOP_divxxxsd,
		    TOP_addxxxss,
		    TOP_addxxxsd,
		    TOP_subxxxss,
		    TOP_subxxxsd,
		    TOP_mulxxxsd,
		    TOP_mulxxxss,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("float unary arithmetic",
		    TOP_sqrtsd,
		    TOP_sqrtss,
		    TOP_rsqrtss,
		    TOP_rcpss,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64, opnd1);

  Instruction_Group("float unary arithmetic extended",
		    TOP_frsqrt128v32,
		    TOP_frcp128v32,
		    TOP_fsqrt128v32,
		    TOP_fsqrt128v64,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);

  Instruction_Group("xmm to xmm",
		    TOP_movdq,
		    TOP_aesmov,
		    TOP_movapd,
		    TOP_movaps,
		    TOP_fmovsldup,
		    TOP_fmovshdup,
		    TOP_fmovddup,
		    /*AVX*/
            TOP_sivmovdqa,
  	        TOP_sivmovdqu,
  	        TOP_sivmovups,
  	        TOP_sivmovupd,
  	        TOP_sivmovapd,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);

  Instruction_Group("x87 load const",
		    TOP_fldz,
		    TOP_UNDEFINED);
  Result( 0, x87 );

  Instruction_Group("x87 to x87",
		    TOP_fmov,
		    TOP_fabs,
		    TOP_fsqrt,
		    TOP_fchs,
		    TOP_frndint,
		    TOP_fcos,
		    TOP_fsin,
		    TOP_UNDEFINED);
  Result(0, x87);
  Operand(0, x87, opnd1);

  Instruction_Group("float madd",
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64, opnd1);
  Operand(1, fp64, opnd2);
  Operand(2, fp64, maddend);

  Instruction_Group("conditional jump",
		    TOP_jb,
		    TOP_jae,
		    TOP_je,
		    TOP_jne,
		    TOP_jbe,
		    TOP_ja,
		    TOP_jge,
		    TOP_jl,
		    TOP_jle,
		    TOP_jg,
		    TOP_jcxz,
		    TOP_jecxz,
		    TOP_jrcxz,
		    TOP_jp,
		    TOP_jnp,
		    TOP_js,
		    TOP_jns,
		    TOP_UNDEFINED);
  Operand(0, rflags,  opnd1);
  Operand(1, pcrel32, target);

  Instruction_Group("pop32",
		    TOP_popl,
		    TOP_UNDEFINED);
  Result(0, int32);
  Result(1, esp);
  Operand(0, int32, opnd1);

  Instruction_Group("pop64",
		    TOP_popq,
		    TOP_UNDEFINED);
  Result(0, int64);
  Result(1, rsp);
  Operand(0, int64, opnd1);

  Instruction_Group("push32",
		    TOP_pushl,
		    TOP_UNDEFINED);
  Result(0, esp);
  Operand(0, int32, opnd1);
  Operand(1, int32, opnd1);

  Instruction_Group("push64",
		    TOP_pushq,
		    TOP_UNDEFINED);
  Result(0, rsp);
  Operand(0, int64, opnd1);
  Operand(1, int64, opnd1);

  Instruction_Group("int8 compare/test with simm",
		    TOP_cmpi8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int8,  opnd1);
  Operand(1, simm32, opnd2);

  Instruction_Group("int16 compare/test with simm",
		    TOP_cmpi16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int16,  opnd1);
  Operand(1, simm32, opnd2);

  Instruction_Group("int32 compare/test with simm",
		    TOP_testi32,
		    TOP_cmpi32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int32,  opnd1);
  Operand(1, simm32, opnd2);

  Instruction_Group("int64 compare/test with simm",
		    TOP_testi64,
		    TOP_cmpi64,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, int64,  opnd1);
  Operand(1, simm32, opnd2);

  Instruction_Group("int compare/test with simm and mem opnd",
		    TOP_cmpxi8,
		    TOP_cmpxi16,
		    TOP_cmpxi32,
		    TOP_UNDEFINED );
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, simm32, opnd2);

  Instruction_Group("int64 compare/test with simm and mem opnd",
		    TOP_cmpxi64,
		    TOP_UNDEFINED );
  Result(0,  rflags);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, simm32, opnd2);

  Instruction_Group("int compare/test with simm and mem opnd scaled-index",
		    TOP_cmpxxi8,
		    TOP_cmpxxi16,
		    TOP_cmpxxi32,
		    TOP_UNDEFINED );
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);
  Operand(4, simm32, opnd2);

  Instruction_Group("int64 compare/test with simm and mem opnd scaled-index",
		    TOP_cmpxxi64,
		    TOP_UNDEFINED );
  Result(0,  rflags);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);
  Operand(4, simm32, opnd2);

  Instruction_Group("int compare/test with simm and mem opnd scaled-index w/o base",
		    TOP_cmpxxxi8,
		    TOP_cmpxxxi16,
		    TOP_cmpxxxi32,
		    TOP_UNDEFINED );
  Result(0,  eflags);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);
  Operand(3, simm32, opnd2);

  Instruction_Group("int64 compare/test with simm and mem opnd scaled-index w/o base",
		    TOP_cmpxxxi64,
		    TOP_UNDEFINED );
  Result(0,  rflags);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);
  Operand(3, simm32, opnd2);

  Instruction_Group("int8 compare/test",
  		    TOP_cmp8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int8, opnd1);
  Operand(1, int8, opnd2);

  Instruction_Group("int16 compare/test",
		    TOP_cmp16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int16, opnd1);
  Operand(1, int16, opnd2);

  Instruction_Group("int32 compare/test",
		    TOP_cmp32,
		    TOP_test32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int32, opnd1);
  Operand(1, int32, opnd2);

  Instruction_Group("int64 compare/test",
		    TOP_cmp64,
		    TOP_test64,
		    TOP_UNDEFINED);
  Result(0,  rflags);
  Operand(0, int64, opnd1);
  Operand(1, int64, opnd2);

   Instruction_Group("int8 compare/test mem opnd",
  		    TOP_cmpx8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int8, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int16 compare/test mem opnd",
		    TOP_cmpx16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int16, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int32 compare/test mem opnd",
		    TOP_cmpx32,
		    TOP_testx32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int32, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int64 compare/test mem opnd",
		    TOP_cmpx64,
		    TOP_testx64,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, int64, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int8 compare/test mem opnd w/ scaled-index",
  		    TOP_cmpxx8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int8, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group("int16 compare/test mem opnd w/ scaled-index",
		    TOP_cmpxx16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int16, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group("int32 compare/test mem opnd w/ scaled-index",
		    TOP_cmpxx32,
		    TOP_testxx32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int32, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group("int64 compare/test mem opnd w/ scaled-index",
		    TOP_cmpxx64,
		    TOP_testxx64,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, int64, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group("int8 compare/test mem opnd w/ scaled-index w/o base",
  		    TOP_cmpxxx8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int8, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("int16 compare/test mem opnd w/ scaled-index w/o base",
		    TOP_cmpxxx16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int16, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("int32 compare/test mem opnd w/ scaled-index w/o base",
		    TOP_cmpxxx32,
		    TOP_testxxx32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int32, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("int64 compare/test mem opnd w/ scaled-index w/o base",
		    TOP_cmpxxx64,
		    TOP_testxxx64,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, int64, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

   Instruction_Group("int8 compare/test mem opnd",
  		    TOP_cmpxr8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int8, opnd1);

  Instruction_Group("int16 compare/test mem opnd",
		    TOP_cmpxr16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int16, opnd1);

  Instruction_Group("int32 compare/test mem opnd",
		    TOP_cmpxr32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int32, opnd1);

  Instruction_Group("int64 compare/test mem opnd",
		    TOP_cmpxr64,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int64, opnd1);

  Instruction_Group("int8 compare/test mem opnd w/ scaled-index",
  		    TOP_cmpxxr8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);
  Operand(4, int8, opnd1);

  Instruction_Group("int16 compare/test mem opnd w/ scaled-index",
		    TOP_cmpxxr16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);
  Operand(4, int16, opnd1);

  Instruction_Group("int32 compare/test mem opnd w/ scaled-index",
		    TOP_cmpxxr32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);
  Operand(4, int32, opnd1);

  Instruction_Group("int64 compare/test mem opnd w/ scaled-index",
		    TOP_cmpxxr64,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);
  Operand(4, int64, opnd1);

  Instruction_Group("int8 compare/test mem opnd w/ scaled-index w/o base",
  		    TOP_cmpxxxr8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);
  Operand(3, int8, opnd1);

  Instruction_Group("int16 compare/test mem opnd w/ scaled-index w/o base",
		    TOP_cmpxxxr16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);
  Operand(3, int16, opnd1);

  Instruction_Group("int32 compare/test mem opnd w/ scaled-index w/o base",
		    TOP_cmpxxxr32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);
  Operand(3, int32, opnd1);

  Instruction_Group("int64 compare/test mem opnd w/ scaled-index w/o base",
		    TOP_cmpxxxr64,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);
  Operand(3, int64, opnd1);

  Instruction_Group("fp ordered compare",
		    TOP_comisd,
		    TOP_comiss,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, fp64, opnd1);
  Operand(1, fp64, opnd2);

  Instruction_Group("fp ordered compare mem opnd",
		    TOP_comixsd,
		    TOP_comixss,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, fp64, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("fp ordered compare mem opnd w/ scaled-index",
		    TOP_comixxsd,
		    TOP_comixxss,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, fp64, opnd1);
  Operand(1, int64, base);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);
  Operand(4, simm32, offset);

  Instruction_Group("fp ordered compare mem opnd w/ scaled-index w/o base",
		    TOP_comixxxsd,
		    TOP_comixxxss,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, fp64, opnd1);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("x87 fp compare",
		    TOP_fucomi,
		    TOP_fucomip,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, x87,  opnd1);
  Operand(1, x87,  opnd2);

  Instruction_Group("fp compare",
		    TOP_cmpss,
		    TOP_cmpsd,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64,  opnd1);
  Operand(1, fp64,  opnd2);
  Operand(2, simm8, opnd3);

  Instruction_Group("fp compare I",
		    TOP_cmpeqss,
		    TOP_cmpltss,
		    TOP_cmpless,
		    TOP_cmpunordss,
		    TOP_cmpneqss,
		    TOP_cmpnltss,
		    TOP_cmpnless,
		    TOP_cmpordss,
		    TOP_cmpeqsd,
		    TOP_cmpltsd,
		    TOP_cmplesd,
		    TOP_cmpunordsd,
		    TOP_cmpneqsd,
		    TOP_cmpnltsd,
		    TOP_cmpnlesd,
		    TOP_cmpordsd,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, fp64,  opnd1);
  Operand(1, fp64,  opnd2);

  Instruction_Group("fp vector compare",
		    TOP_cmpps,
		    TOP_cmppd,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128,  opnd1);
  Operand(1, fp128,  opnd2);
  Operand(2, simm8, opnd3);

  Instruction_Group("fp vector compare I",
		    TOP_cmpeqps,
		    TOP_cmpeqpd,
		    TOP_cmpltpd,
		    TOP_cmplepd,
		    TOP_cmpgtpd,
		    TOP_cmpgepd,
		    TOP_cmpneqpd,
		    TOP_cmpnltpd,
		    TOP_cmpnlepd,
		    TOP_cmpngtpd,
		    TOP_cmpngepd,
		    TOP_cmpordpd,
		    TOP_cmpunordpd,
		    TOP_cmpltps,
		    TOP_cmpleps,
		    TOP_cmpunordps,
		    TOP_cmpneqps,
		    TOP_cmpnltps,
		    TOP_cmpnleps,
		    TOP_cmpordps,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128,  opnd1);
  Operand(1, fp128,  opnd2);


  Instruction_Group("load32 effective addr",
		    TOP_lea32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

	Instruction_Group("Blending condition",
				TOP_pblendw,
				TOP_blendpd,
				TOP_blendps,
				TOP_pcmpestri,
				TOP_pcmpestrm,
				TOP_pcmpistri,
				TOP_pcmpistrm,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, fp128, opnd2);
	Operand(2, simm8, opnd3);

	Instruction_Group("Blending condition x",
				TOP_pblendwx,
				TOP_blendpdx,
				TOP_blendpsx,
				TOP_dppdx,
				TOP_dppsx,
				TOP_insertpsx,
				TOP_mpsadbwx,
				TOP_pcmpestrix,
				TOP_pcmpestrmx,
				TOP_pcmpistrix,
				TOP_pcmpistrmx,
				TOP_pinsrbx,
				TOP_pinsrdx,
				TOP_pinsrqx,
				TOP_roundpdx,
				TOP_roundpsx,
				TOP_roundsdx,
				TOP_roundssx,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, simm32,offset);
	Operand(3, simm8, opnd3);

	Instruction_Group("Blending condition xx",
				TOP_pblendwxx,
				TOP_blendpdxx,
				TOP_blendpsxx,
				TOP_dppdxx,
				TOP_dppsxx,
				TOP_insertpsxx,
				TOP_mpsadbwxx,
				TOP_pcmpestrixx,
				TOP_pcmpestrmxx,
				TOP_pcmpistrixx,
				TOP_pcmpistrmxx,
				TOP_pinsrbxx,
				TOP_pinsrdxx,
				TOP_pinsrqxx,
				TOP_roundpdxx,
				TOP_roundpsxx,
				TOP_roundsdxx,
				TOP_roundssxx,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, int64, index);
	Operand(3, uimm8, scale);
	Operand(4, simm32, offset);
	Operand(5, simm8, opnd3);

	Instruction_Group("Blending condition xxx",
				TOP_pblendwxxx,
				TOP_blendpdxxx,
				TOP_blendpsxxx,
				TOP_dppdxxx,
				TOP_dppsxxx,
				TOP_insertpsxxx,
				TOP_mpsadbwxxx,
				TOP_pcmpestrixxx,
				TOP_pcmpestrmxxx,
				TOP_pcmpistrixxx,
				TOP_pcmpistrmxxx,
				TOP_pinsrbxxx,
				TOP_pinsrdxxx,
				TOP_pinsrqxxx,
				TOP_roundpdxxx,
				TOP_roundpsxxx,
				TOP_roundsdxxx,
				TOP_roundssxxx,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, uimm8, scale);
	Operand(3, simm32,offset);
	Operand(4, simm8, opnd3);



	Instruction_Group("Blending condition no imm x",
				TOP_pblendvbx,
				TOP_blendvpdx,
				TOP_blendvpsx,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128,opnd1);
	Operand(1, int64, base);
	Operand(2, simm32,offset);
	Operand(3, xmm0);

	Instruction_Group("Blending condition no imm ",
				TOP_pblendvb,
				TOP_blendvpd,
				TOP_blendvps,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128,opnd1);
	Operand(1, fp128, opnd2);
	Operand(2, xmm0);

	Instruction_Group("Blending condition no imm xx",
				TOP_pblendvbxx,
				TOP_blendvpdxx,
				TOP_blendvpsxx,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128,opnd1);
	Operand(1, int64, base);
	Operand(2, int64, index);
	Operand(3, uimm8, scale);
	Operand(4, simm32, offset);
	Operand(5, xmm0);

	Instruction_Group("Blending condition no imm  xxx",
				TOP_pblendvbxxx,
				TOP_blendvpdxxx,
				TOP_blendvpsxxx,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128,opnd1);
	Operand(1, int64, base);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);
	Operand(2, xmm0);
	
	Instruction_Group("round",
				TOP_roundpd,
				TOP_roundsd,
				TOP_roundps,
				TOP_roundss,
				TOP_dppd,
				TOP_dpps,
				TOP_mpsadbw,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128,opnd1);
	Operand(1, fp128, opnd2);
	Operand(2, simm8, opnd3);

	Instruction_Group("pmul128",
				TOP_pmuldq,
				TOP_pmulld,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128,opnd1);
	Operand(1, fp128, opnd2);

	Instruction_Group("pextr in32",
				TOP_pextrb,
				TOP_pextrd,
				TOP_UNDEFINED);
	Result(0,int32);
	Operand(0,int32,opnd1);
	Operand(1,fp128,opnd2);
	Operand(2,uimm8,opnd3);

	Instruction_Group("pinsr in32",
				TOP_pinsrb,
				TOP_pinsrd,
				TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int32, opnd2);
	Operand(2, simm8, opnd3);



	Instruction_Group("pextr in64",
				TOP_pextrq,
				TOP_UNDEFINED);
	Result(0,int64);
	Operand(0,int64,opnd1);
	Operand(1,fp128,opnd2);
	Operand(2,uimm8,opnd3);
	
	Instruction_Group("pinsr in64",
				TOP_pinsrq,
				TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, opnd2);
	Operand(2, simm8, opnd3);


	Instruction_Group("extractps fp32",
				TOP_extractps,
				TOP_UNDEFINED);
	Result(0,int64);
	Operand(0,fp128,opnd1);
	Operand(1,uimm8,opnd2);

	Instruction_Group("extractps fp32 x",
				TOP_extractpsx,
				TOP_pextrbx,
				TOP_pextrdx,
				TOP_pextrqx,
				TOP_pextrwx,
				TOP_UNDEFINED);
	Operand(0, fp128, opnd1);
    Operand(1, simm8, opnd2);
    Operand(2, int64, base);
    Operand(3, simm32, offset);

	Instruction_Group("extractps fp32 xx",
				TOP_extractpsxx,
				TOP_pextrbxx,
				TOP_pextrdxx,
				TOP_pextrqxx,
				TOP_pextrwxx,
				TOP_UNDEFINED);
	Operand(0, fp128, opnd1);
    Operand(1, simm8, opnd2);
    Operand(2, int64, base);
    Operand(3, simm32, offset);
    Operand(4, int64, index);
    Operand(5, uimm8, scale);

	Instruction_Group("extractps fp32 xxx",
				TOP_extractpsxxx,
				TOP_pextrbxxx,
				TOP_pextrdxxx,
				TOP_pextrqxxx,
				TOP_pextrwxxx,
				TOP_UNDEFINED);
	Operand(0, fp128, opnd1);
	Operand(1, simm8, opnd2);
	Operand(2, int64, index);
	Operand(3, uimm8, scale);
	Operand(4, simm32, offset);


	Instruction_Group("insertps fp32",
				TOP_insertps,
				TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0,fp128,opnd1);
	Operand(1,fp128,opnd2);
	Operand(2,simm32,opnd3);



	Instruction_Group("ssse3 instructions 128",
						TOP_phaddw128,
						TOP_phaddd128,
						TOP_phaddsw128,
						TOP_phsubw128,
						TOP_phsubd128,
						TOP_phsubsw128,
						TOP_pmaddubsw128,
						TOP_pmulhrsw128,
						TOP_pshufb128,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, fp128, opnd1);
	Operand(1, fp128, opnd2);

	Instruction_Group("ssse3 instructions 128x",
						TOP_phaddwx128,
						TOP_phadddx128,
						TOP_phaddswx128,
						TOP_phsubwx128,
						TOP_phsubdx128,
						TOP_phsubswx128,
						TOP_pmaddubswx128,
						TOP_pmulhrswx128,
						TOP_pshufbx128,
						/*some sse4 suitable for this*/
		                TOP_packusdwx,
		                TOP_pmaxsbx,
		                TOP_pmaxsdx,
		                TOP_pmaxudx,
		                TOP_pmaxuwx,
		                TOP_pminsbx,
		                TOP_pminsdx,
		                TOP_pminudx,
		                TOP_pminuwx,
		                TOP_pmuldqx,
						TOP_pmulldx,
						TOP_pcmpeqqx,
					    TOP_pcmpgtqx,
					    TOP_phminposuwx,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, simm32, offset);

	Instruction_Group("ssse3 instructions 128xx",
						TOP_phaddwxx128,
						TOP_phadddxx128,
						TOP_phaddswxx128,
						TOP_phsubwxx128,
						TOP_phsubdxx128,
						TOP_phsubswxx128,
						TOP_pmaddubswxx128,
						TOP_pmulhrswxx128,
						TOP_pshufbxx128,
						/*some sse4 suitable for this*/
	                    TOP_packusdwxx,
	                    TOP_pmaxsbxx,
		                TOP_pmaxsdxx,
		                TOP_pmaxudxx,
		                TOP_pmaxuwxx,
		                TOP_pminsbxx,
		                TOP_pminsdxx,
		                TOP_pminudxx,
		                TOP_pminuwxx,
		                TOP_pmuldqxx,
						TOP_pmulldxx,
						TOP_pcmpeqqxx,
					    TOP_pcmpgtqxx,
					    TOP_phminposuwxx,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, int64, index);
	Operand(3, uimm8, scale);
	Operand(4, simm32, offset);

	Instruction_Group("ssse3 instructions 128xxx",
						TOP_phaddwxxx128,
						TOP_phadddxxx128,
						TOP_phaddswxxx128,
						TOP_phsubwxxx128,
						TOP_phsubdxxx128,
						TOP_phsubswxxx128,
						TOP_pmaddubswxxx128,
						TOP_pmulhrswxxx128,
						TOP_pshufbxxx128,
						/*some sse4 suitable for this*/
	                    TOP_packusdwxxx,
	                    TOP_pmaxsbxxx,
		                TOP_pmaxsdxxx,
		                TOP_pmaxudxxx,
		                TOP_pmaxuwxxx,
		                TOP_pminsbxxx,
		                TOP_pminsdxxx,
		                TOP_pminudxxx,
		                TOP_pminuwxxx,
		                TOP_pmuldqxxx,
						TOP_pmulldxxx,
						TOP_pcmpeqqxxx,
					    TOP_pcmpgtqxxx,
					    TOP_phminposuwxxx,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);
	
	Instruction_Group("ssse3 instructions 64",
						TOP_phaddw,
						TOP_phaddd,
						TOP_phaddsw,
						TOP_phsubw,
						TOP_phsubd,
						TOP_phsubsw,
						TOP_pmaddubsw,
						TOP_pmulhrsw,
						TOP_pshufb,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, mmx, opnd1);
	Operand(1, mmx, opnd2);

	Instruction_Group("ssse3 instructions 64kx",
						TOP_phaddwx,
						TOP_phadddx,
						TOP_phaddswx,
						TOP_phsubwx,
						TOP_phsubdx,
						TOP_phsubswx,
						TOP_pmaddubswx,
						TOP_pmulhrswx,
						TOP_pshufbx,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, mmx, opnd1);
	Operand(1, int64, base);
	Operand(2, simm32, offset);

	Instruction_Group("ssse3 instructions 64xx",
						TOP_phaddwxx,
						TOP_phadddxx,
						TOP_phaddswxx,
						TOP_phsubwxx,
						TOP_phsubdxx,
						TOP_phsubswxx,
						TOP_pmaddubswxx,
						TOP_pmulhrswxx,
						TOP_pshufbxx,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, mmx, opnd1);
	Operand(1, int64, base);
	Operand(2, int64, index);
	Operand(3, uimm8, scale);
	Operand(4, simm32, offset);

	Instruction_Group("ssse3 instructions 64xxx",
						TOP_phaddwxxx,
						TOP_phadddxxx,
						TOP_phaddswxxx,
						TOP_phsubwxxx,
						TOP_phsubdxxx,
						TOP_phsubswxxx,
						TOP_pmaddubswxxx,
						TOP_pmulhrswxxx,
						TOP_pshufbxxx,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, mmx, opnd1);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);
	
	Instruction_Group("ssse3 instructions 128 one operand",
						TOP_pabsb128,
						TOP_pabsw128,
						TOP_pabsd128,
						TOP_psignb128,
					    TOP_psignw128,
					    TOP_psignd128,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, fp128, opnd1);

	Instruction_Group("ssse3 instructions 128 one operandx",
						TOP_pabsbx128,
						TOP_pabswx128,
						TOP_pabsdx128,
	                    TOP_psignbx128,
					    TOP_psignwx128,
					    TOP_psigndx128,
					    /*SSE4 suitable*/
					    TOP_pmovsxbwx,
						TOP_pmovsxbdx,
						TOP_pmovsxbqx,
						TOP_pmovsxwdx,
						TOP_pmovsxwqx,
						TOP_pmovsxdqx,
						TOP_pmovzxbwx,
						TOP_pmovzxbdx,
						TOP_pmovzxbqx,
						TOP_pmovzxwdx,
						TOP_pmovzxwqx,
						TOP_pmovzxdqx,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, int64, base);
	Operand(1, simm32, offset);

	Instruction_Group("ssse3 instructions 128 one operandxx",
						TOP_pabsbxx128,
						TOP_pabswxx128,
						TOP_pabsdxx128,
						TOP_psignbxx128,
					    TOP_psignwxx128,
					    TOP_psigndxx128,
					    /*SSE4 suitable*/
					     TOP_pmovsxbwxx,
						TOP_pmovsxbdxx,
						TOP_pmovsxbqxx,
						TOP_pmovsxwdxx,
						TOP_pmovsxwqxx,
						TOP_pmovsxdqxx,
						TOP_pmovzxbwxx,
						TOP_pmovzxbdxx,
						TOP_pmovzxbqxx,
						TOP_pmovzxwdxx,
						TOP_pmovzxwqxx,
						TOP_pmovzxdqxx,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, int64, base);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);

	Instruction_Group("ssse3 instructions 128 one operandxxx",
						TOP_pabsbxxx128,
						TOP_pabswxxx128,
						TOP_pabsdxxx128,
						TOP_psignbxxx128,
					    TOP_psignwxxx128,
					    TOP_psigndxxx128,
					    /*SSE4 suitable*/
					     TOP_pmovsxbwxxx,
						TOP_pmovsxbdxxx,
						TOP_pmovsxbqxxx,
						TOP_pmovsxwdxxx,
						TOP_pmovsxwqxxx,
						TOP_pmovsxdqxxx,
						TOP_pmovzxbwxxx,
						TOP_pmovzxbdxxx,
						TOP_pmovzxbqxxx,
						TOP_pmovzxwdxxx,
						TOP_pmovzxwqxxx,
						TOP_pmovzxdqxxx,
						TOP_UNDEFINED);
	Result(0,fp128);
	Operand(0, int64, base);
	Operand(1, uimm8, scale);
	Operand(2, simm32, offset);
	
	Instruction_Group("ssse3 instructions 64 one operand",
						TOP_pabsb,
						TOP_pabsw,
						TOP_pabsd,
						TOP_psignb,
					    TOP_psignw,
					    TOP_psignd,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, mmx, opnd1);

	Instruction_Group("ssse3 instructions 64 one operand x",
						TOP_pabsbx,
						TOP_pabswx,
						TOP_pabsdx,
						TOP_psignbx,
					    TOP_psignwx,
					    TOP_psigndx,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, int64, base);
	Operand(1, simm32, offset);

	Instruction_Group("ssse3 instructions 64 one operand xx",
						TOP_pabsbxx,
						TOP_pabswxx,
						TOP_pabsdxx,
						TOP_psignbxx,
					    TOP_psignwxx,
					    TOP_psigndxx,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, int64, base);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);

	Instruction_Group("ssse3 instructions 64 one operand xxx",
						TOP_pabsbxxx,
						TOP_pabswxxx,
						TOP_pabsdxxx,
						TOP_psignbxxx,
					    TOP_psignwxxx,
					    TOP_psigndxxx,
						TOP_UNDEFINED);
	Result(0,mmx);
	Operand(0, int64, base);
	Operand(1, uimm8, scale);
	Operand(2, simm32, offset);

	Instruction_Group("ssse3 two fp128 and a const",
						TOP_palignr128,
						TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, fp128, opnd2);
	Operand(2, simm8, opnd3);

	Instruction_Group("ssse3 two fp128 and a const x",
						TOP_palignrx128,
						TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, simm32, offset);
	Operand(3, simm8, opnd2);

	Instruction_Group("ssse3 two fp128 and a const xx",
						TOP_palignrxx128,
						TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, int64, index);
	Operand(3, uimm8, scale);
	Operand(4, simm32, offset);
	Operand(5, simm8, opnd2);

	Instruction_Group("ssse3 two fp128 and a const xxx",
						TOP_palignrxxx128,
						TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);
	Operand(4, simm8, opnd2);
	
	Instruction_Group("ssse3 two mmx and a const",
						TOP_palignr,
						TOP_UNDEFINED);
	Result(0, mmx);
	Operand(0, mmx, opnd1);
	Operand(1, mmx, opnd2);
	Operand(2, simm8, opnd3);

	Instruction_Group("ssse3 two mmx and a const x",
						TOP_palignrx,
						TOP_UNDEFINED);
	Result(0, mmx);
	Operand(0, mmx, opnd1);
	Operand(1, int64, base);
	Operand(2, simm32, offset);
	Operand(3, simm8, opnd3);

	Instruction_Group("ssse3 two mmx and a const xx",
						TOP_palignrxx,
						TOP_UNDEFINED);
	Result(0, mmx);
	Operand(0, mmx, opnd1);
	Operand(1, int64, base);
	Operand(2, int64, index);
	Operand(3, uimm8, scale);
	Operand(4, simm32, offset);
	Operand(5, simm8, opnd3);

	Instruction_Group("ssse3 two mmx and a const xxx",
						TOP_palignrxxx,
						TOP_UNDEFINED);
	Result(0, mmx);
	Operand(0, mmx, opnd1);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);
	Operand(4, simm8, opnd3);

   Instruction_Group("qi vector compare explicit length retruns index",
		    TOP_pcmpestriintr,
		    TOP_UNDEFINED);
  Result(0, ecx);
  Result(1, rflags);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);
  Operand(2, simm8, opnd3);
  Operand(3, eax, opnd4);
  Operand(4, edx, opnd5);

   Instruction_Group("qi vector compare explicit length retruns mask",
		    TOP_pcmpestrmintr,
		    TOP_UNDEFINED);
  Result(0, xmm0);
  Result(1, rflags);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);
  Operand(2, simm8, opnd3);
  Operand(3, eax, opnd4);
  Operand(4, edx, opnd5);

  Instruction_Group("qi vector compare implicit length returns index",
          TOP_pcmpistriintr,      
          TOP_UNDEFINED);
  Result(0, ecx);
  Result(1, rflags);
  Operand(0, fp128,  opnd1);
  Operand(1, fp128,  opnd2);
  Operand(2, simm8, opnd3);

  Instruction_Group("qi vector compare implicit length returns mask",
            TOP_pcmpistrmintr,
		    TOP_UNDEFINED);
  Result(0, xmm0);
  Result(1, rflags);
  Operand(0, fp128,  opnd1);
  Operand(1, fp128,  opnd2);
  Operand(2, simm8, opnd3);

	Instruction_Group("qi vector compare pcmpgtq",
				TOP_pcmpgtq,
				TOP_UNDEFINED);
	Result(0, fp128);
	Operand(0, fp128, opnd1);
	Operand(1, fp128, opnd2);


  Instruction_Group("crc sse4.2 crc32b",
		  	TOP_crc32b,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32);
  Operand(1, int8);

  Instruction_Group("crc sse4.2 crc32w",
		  	TOP_crc32w,
		  	TOP_crc32,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32);
  Operand(1, int16);

  Instruction_Group("crc sse4.2 crc32l",
		  	TOP_crc32l,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32);
  Operand(1, int32);

  Instruction_Group("crc sse4.2 crc32q",
		  	TOP_crc32q,
		  	TOP_crc32_64,
			TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64);
  Operand(1, int64);

  Instruction_Group("crc sse4.2 crc32 x",
		  	TOP_crc32x,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32);
	 Operand(1, int64, base);
	 Operand(2, simm32, offset);

  Instruction_Group("crc sse4.2 crc32 xx",
		  	TOP_crc32xx,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32);
	 Operand(1, int64, base);
	 Operand(2, int64, index);
	 Operand(3, uimm8, scale);
	 Operand(4, simm32, offset);
		
  Instruction_Group("crc sse4.2 crc32 xxx",
		  	TOP_crc32xxx,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32);
	 Operand(1, int64, index);
	 Operand(2, uimm8, scale);
	 Operand(3, simm32, offset);

  Instruction_Group("crc sse4.2 crc64 x",
		  	TOP_crc32_64x,
			TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64);
	 Operand(1, int64, base);
	 Operand(2, simm32, offset);

  Instruction_Group("crc sse4.2 crc64 xx",
		  	TOP_crc32_64xx,
			TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64);
	 Operand(1, int64, base);
	 Operand(2, int64, index);
	 Operand(3, uimm8, scale);
	 Operand(4, simm32, offset);
		
  Instruction_Group("crc sse4.2 crc64 xxx",
		  	TOP_crc32_64xxx,
			TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64);
	 Operand(1, int64, index);
	 Operand(2, uimm8, scale);
	 Operand(3, simm32, offset);

  Instruction_Group("load64 effective addr",
		    TOP_lea64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  
	Instruction_Group(" sse4.2 popcntl",
		  	TOP_popcntl,
			TOP_UNDEFINED);
  Result(0, edx);
  Operand(0, int32 );
	
	Instruction_Group(" sse4.2 popcntq",
		  	TOP_popcntq,
			TOP_UNDEFINED);
  Result(0, rdx);
  Operand(0, int64);

  	Instruction_Group(" sse4.2 popcnt 16 x",
		  	TOP_popcnt16x,
			TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
	
	Instruction_Group(" sse4.2 popcnt 32 x",
		  	TOP_popcnt32x,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group(" sse4.2 popcnt 64 x",
		  	TOP_popcnt64x,
			TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group(" sse4.2 popcnt 16 xx",
		  	TOP_popcnt16xx,
			TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);
	
	Instruction_Group(" sse4.2 popcnt 32 xx",
		  	TOP_popcnt32xx,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group(" sse4.2 popcnt 64 xx",
		  	TOP_popcnt64xx,
			TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group(" sse4.2 popcnt 16 xxx",
		  	TOP_popcnt16xxx,
			TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int64, base);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);
	
	Instruction_Group(" sse4.2 popcnt 32 xxx",
		  	TOP_popcnt32xxx,
			TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, base);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group(" sse4.2 popcnt 64 xxx",
		  	TOP_popcnt64xxx,
			TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, base);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("load32 effective addr w/ indx",
		    TOP_leax32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("load64 effective addr w/ indx",
		    TOP_leax64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("load32 effective addr w/ indx w/o base",
		    TOP_leaxx32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("load64 effective addr w/ indx w/o base",
		    TOP_leaxx64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("int32 load w/ indx",
		    TOP_ldx32,
		    TOP_ldx8_32,
		    TOP_ldxu8_32,
		    TOP_ldx16_32,
		    TOP_ldxu16_32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);

  Instruction_Group("int64 load w/ indx",
		    TOP_ldx64,
		    TOP_ldx8_64,
		    TOP_ldxu8_64,
		    TOP_ldx16_64,
		    TOP_ldxu16_64,
		    TOP_ldx32_64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);

  Instruction_Group("int32 load w/ indx w/o base",
		    TOP_ldxx32,
		    TOP_ldxx8_32,
		    TOP_ldxxu8_32,
		    TOP_ldxx16_32,
		    TOP_ldxxu16_32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("int64 load w/ indx w/o base",
		    TOP_ldxx64,
		    TOP_ldxx8_64,
		    TOP_ldxxu8_64,
		    TOP_ldxx16_64,
		    TOP_ldxxu16_64,
		    TOP_ldxx32_64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("int8 store w/ indx",
		    TOP_storex8,
		    TOP_UNDEFINED);
  Operand(0, int8, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  Operand(3, int64, index);
  Operand(4, uimm8, scale);

  Instruction_Group("int16 store w/ indx",
		    TOP_storex16,
		    TOP_UNDEFINED);
  Operand(0, int16, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  Operand(3, int64, index);
  Operand(4, uimm8, scale);

  Instruction_Group("int32 store w/ indx",
		    TOP_storex32,
		    TOP_storentix32,
		    TOP_UNDEFINED);
  Operand(0, int32, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  Operand(3, int64, index);
  Operand(4, uimm8, scale);

  Instruction_Group("int64 store w/ indx",
		    TOP_storex64,
		    TOP_storentix64,
		    TOP_UNDEFINED);
  Operand(0, int64, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  Operand(3, int64, index);
  Operand(4, uimm8, scale);

  Instruction_Group("int8 store w/ indx w/o base",
		    TOP_storexx8,
		    TOP_UNDEFINED);
  Operand(0, int8, storeval);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("int16 store w/ indx w/o base",
		    TOP_storexx16,
		    TOP_UNDEFINED);
  Operand(0, int16, storeval);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("int32 store w/ indx w/o base",
		    TOP_storexx32,
		    TOP_storentixx32,
		    TOP_UNDEFINED);
  Operand(0, int32, storeval);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("int64 store w/ indx w/o base",
		    TOP_storexx64,
		    TOP_storentixx64,
		    TOP_UNDEFINED);
  Operand(0, int64, storeval);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("fp load w/ indx",
		    TOP_ldssx,
		    TOP_ldsdx,
		    TOP_UNDEFINED);
  Result(0,  fp64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);

  Instruction_Group("fp convert w/ indx",
		    TOP_cvtsd2ss_xx,
		    TOP_cvtsi2sd_xx,
		    TOP_cvtsi2ss_xx,
		    TOP_cvtsi2sdq_xx,
		    TOP_cvtsi2ssq_xx,
		    TOP_UNDEFINED);
  Result(0,  fp64);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("fp load vector w/ indx",
		    TOP_lddqax,
		    TOP_lddqux,
		    TOP_ldhpsx,
		    TOP_ldlpsx,
		    TOP_ldhpdx,
		    TOP_ldlpdx,
		    TOP_ldapdx,
		    TOP_ldapsx,
		    TOP_fmovsldupxx,
		    TOP_fmovshdupxx,
		    TOP_fmovddupxx,
		    TOP_UNDEFINED);
  Result(0,  fp128);
  Operand(0, int64, base);
  Operand(1, simm32, offset);
  Operand(2, int64, index);
  Operand(3, uimm8, scale);

  Instruction_Group("fp load w/ indx w/o base",
		    TOP_ldssxx,
		    TOP_ldsdxx,
		    TOP_UNDEFINED);
  Result(0,  fp64);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("fp convert w/ indx w/o base",
		    TOP_cvtsd2ss_xxx,
		    TOP_cvtsi2sd_xxx,
		    TOP_cvtsi2ss_xxx,
		    TOP_cvtsi2sdq_xxx,
		    TOP_cvtsi2ssq_xxx,
		    TOP_UNDEFINED);
  Result(0,  fp64);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("fp load vector w/ indx w/o base",
		    TOP_lddqaxx,
		    TOP_lddquxx,
		    TOP_ldhpsxx,
		    TOP_ldlpsxx,
		    TOP_ldhpdxx,
		    TOP_ldlpdxx,
		    TOP_ldapdxx,
		    TOP_ldapsxx,
		    TOP_fmovsldupxxx,
		    TOP_fmovshdupxxx,
		    TOP_fmovddupxxx,
		    TOP_UNDEFINED);
  Result(0,  fp128);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("fp store w/ indx",
		    TOP_stssx,
		    TOP_stntssx,
		    TOP_stsdx,
		    TOP_stntsdx,
		    TOP_UNDEFINED);
  Operand(0, fp64, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  Operand(3, int64, index);
  Operand(4, uimm8, scale);

  Instruction_Group("fp store vector w/ indx",
		    TOP_stdqax,
		    TOP_stdqux,
		    TOP_sthpdx,
		    TOP_stlpdx,
		    TOP_sthpsx,
		    TOP_stlpsx,
		    TOP_stapsx,
		    TOP_stapdx,
		    TOP_stntpdx,
		    TOP_stntpsx,
		    TOP_UNDEFINED);
  Operand(0, fp128, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);
  Operand(3, int64, index);
  Operand(4, uimm8, scale);

  Instruction_Group("fp store w/ indx w/o base",
		    TOP_stssxx,
		    TOP_stntssxx,
		    TOP_stsdxx,
		    TOP_stntsdxx,
		    TOP_UNDEFINED);
  Operand(0, fp64, storeval);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("fp store vector w/ indx w/o base",
		    TOP_stdqaxx,
		    TOP_stdquxx,
		    TOP_sthpdxx,
		    TOP_stlpdxx,
		    TOP_sthpsxx,
		    TOP_stlpsxx,
		    TOP_stapsxx,
		    TOP_stapdxx,
		    TOP_stntpdxx,
		    TOP_stntpsxx,
		    TOP_UNDEFINED);
  Operand(0, fp128, storeval);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("jump",
		    TOP_jmp,
		    TOP_call,
		    TOP_UNDEFINED);
  Operand(0, pcrel32, target);

  Instruction_Group("jump register",
		    TOP_ijmp,
		    TOP_icall,
		    TOP_UNDEFINED);
  Operand(0, int64, target);


  Instruction_Group("jump mem",
		    TOP_ijmpx,
		    TOP_icallx,
		    TOP_stmxcsr,
		    TOP_ldmxcsr,
		    TOP_UNDEFINED);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("jump mem w/ scaled index",
		    TOP_ijmpxx,
		    TOP_icallxx,
		    TOP_UNDEFINED);
  Operand(0, int64, base);
  Operand(1, int64, index);
  Operand(2, uimm8, scale);
  Operand(3, simm32, offset);

  Instruction_Group("jump mem w/ scaled index w/o base",
		    TOP_ijmpxxx,
		    TOP_icallxxx,
		    TOP_UNDEFINED);
  Operand(0, int64, index);
  Operand(1, uimm8, scale);
  Operand(2, simm32, offset);

  Instruction_Group("label",
		    TOP_label,
		    TOP_UNDEFINED);
  Operand(0, pcrel32);
  Relocatable(0);

  Instruction_Group("spadjust",
		    TOP_spadjust,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, simm16, opnd2);

  Instruction_Group("savexmms",
	  	    TOP_savexmms,
		    TOP_UNDEFINED);
  Operand(0, rax, opnd1); 
  Operand(1, uimm8);	// gives number of xmm parameters left in vararg part
  Operand(2, int64, base);  
  Operand(3, simm32, offset); // offset for the first save
  Operand(4, pcrel32, target); // label after the savexmms
  Operand(5, r11);

  Instruction_Group("intrncall",
		    TOP_intrncall,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, uimm16);	// the intrinsic ID

  Instruction_Group("pregtn tuple",
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, uimm16);

  Instruction_Group("leave",
		    TOP_leave,
		    TOP_UNDEFINED);
  Result(0, rsp);
  Operand(0, rbp, opnd1);

  Instruction_Group("zero32",
		    TOP_zero32,
		    TOP_UNDEFINED);
  Result(0, int32);

  Instruction_Group("zero64",
		    TOP_zero64,
		    TOP_UNDEFINED);
  Result(0, int64);

  Instruction_Group("xzero",
		    TOP_xzero32,
		    TOP_xzero64,
		    TOP_UNDEFINED);
  Result(0, fp64);

  Instruction_Group("xzerov",
		    TOP_xzero128v32,
		    TOP_xzero128v64,
		    /*avx added by hand*/
            TOP_vxzero128v64,
            TOP_vxzero128v32,
            TOP_vxzero256v64,
            TOP_vxzero256v32,
		    TOP_UNDEFINED);
  Result(0, fp128);

  Instruction_Group( "int8 arithmetic lock",
		     TOP_lock_add8,
		     TOP_lock_and8,
		     TOP_lock_or8,
		     TOP_lock_xor8,
		     TOP_lock_sub8,
		     TOP_UNDEFINED);
  Operand(0, int8, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "int16 arithmetic lock",
		     TOP_lock_add16,
		     TOP_lock_and16,
		     TOP_lock_or16,
		     TOP_lock_xor16,
		     TOP_lock_sub16,
		     TOP_UNDEFINED);
  Operand(0, int16, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "int32 arithmetic lock",
		     TOP_lock_add32,
		     TOP_lock_adc32,
		     TOP_lock_and32,
		     TOP_lock_or32,
		     TOP_lock_xor32,
		     TOP_lock_sub32,
		     TOP_UNDEFINED);
  Operand(0, int32, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "int64 arithmetic lock",
		     TOP_lock_add64,
		     TOP_lock_and64,
		     TOP_lock_or64,
		     TOP_lock_xor64,
		     TOP_lock_sub64,
		     TOP_UNDEFINED);
  Operand(0, int64, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int8 compare & exchange mem opnd lock",
		    TOP_lock_cmpxchg8,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Result(1,  eax);
  Operand(0, eax, opnd1);
  Operand(1, int8, opnd2);
  Operand(2, int64, base);
  Operand(3, simm32, offset);

  Instruction_Group("int16 compare & exchange mem opnd lock",
		    TOP_lock_cmpxchg16,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Result(1,  eax);
  Operand(0, eax, opnd1);
  Operand(1, int16, opnd2);
  Operand(2, int64, base);
  Operand(3, simm32, offset);

  Instruction_Group("int32 compare & exchange mem opnd lock",
		    TOP_lock_cmpxchg32,
		    TOP_UNDEFINED);
  Result(0,  eflags);
  Result(1,  eax);
  Operand(0, eax, opnd1);
  Operand(1, int32, opnd2);
  Operand(2, int64, base);
  Operand(3, simm32, offset);

  Instruction_Group("int64 compare & exchange mem opnd lock",
		    TOP_lock_cmpxchg64,
		    TOP_UNDEFINED);
  Result(0,  rflags);
  Result(1,  eax);
  Operand(0, rax, opnd1);
  Operand(1, int64, opnd2);
  Operand(2, int64, base);
  Operand(3, simm32, offset);

  Instruction_Group("int8 exchange arithmetic lock",
		    TOP_lock_xadd8,
		    TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int16 exchange arithmetic lock",
		    TOP_lock_xadd16,
		    TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int32 exchange arithmetic lock",
		    TOP_lock_xadd32,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int64 exchange arithmetic lock",
		    TOP_lock_xadd64,
		    TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group( "bsf32",
                     TOP_bsf32,
                     TOP_bsr32,
                     TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);

  Instruction_Group( "bsf64",
                     TOP_bsf64,
                     TOP_bsr64,
                     TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);

  Instruction_Group("int64 load to mmx",
		    TOP_ld64_2m,
		    TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("int64 load to mmx w/o base or index",
		    TOP_ld64_2m_n32,
		    TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, simm32, offset);

  Instruction_Group("int64 store from mmx",
		    TOP_store64_fm,
		    TOP_storent64_fm,
		    TOP_UNDEFINED);
  Operand(0, mmx, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int64 store from mmx w/o base or index",
		    TOP_store64_fm_n32,
		    TOP_UNDEFINED);
  Operand(0, mmx, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("int64 mov between mmx",
		    TOP_mov64_m,
		    TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, mmx, opnd1);

  Instruction_Group("mmx packed move",
                    TOP_pmovmskb,
                    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, mmx, opnd1);

  Instruction_Group("sse packed move",
                    TOP_pmovmskb128,
                    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, fp128, opnd1);
  
	Instruction_Group("sse4.1 movngdqa move",
										TOP_movntdqa,
                    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, int64, opnd1);

  Instruction_Group("mmx packed word modification",
		    TOP_pextrw64,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, mmx, opnd1);
  Operand(1, uimm8, opnd2);

  Instruction_Group("xmm packed word modification",
		    TOP_pextrw128,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, fp128, opnd1);
  Operand(1, uimm8, opnd2);

  Instruction_Group("mmx packed word insertion",
		    TOP_pinsrw64,
		    TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, mmx, opnd1);
  Operand(1, int32, opnd2);
  Operand(2, uimm8, opnd3);

  Instruction_Group("xmm packed word insertion",
		    TOP_pinsrw128,
		    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, int32, opnd2);
  Operand(2, uimm8, opnd3);

  Instruction_Group( "mmx vector arithmetic",
		     TOP_add64v8,
		     TOP_add64v16,
		     TOP_add64v32,
		     TOP_sub64v8,
		     TOP_sub64v16,
		     TOP_sub64v32,
		     TOP_paddsb,
		     TOP_paddsw,
		     TOP_paddq,
		     TOP_psubsb,
		     TOP_psubsw,
		     TOP_paddusb,
		     TOP_paddusw,
		     TOP_psubusb,
		     TOP_psubusw,
		     TOP_pmullw,
		     TOP_pmulhw,
		     TOP_pmuludq,
		     TOP_pmaddwd,
		     TOP_pcmpeqb,
		     TOP_pcmpeqw,
		     TOP_pcmpeqd,
		     TOP_pcmpgtb,
		     TOP_pcmpgtw,
		     TOP_pcmpgtd,
		     TOP_punpckhbw,
		     TOP_punpckhwd,
		     TOP_punpckhdq,
		     TOP_punpckl64v8,
		     TOP_punpckl64v16,
		     TOP_punpckl64v32,
		     TOP_packsswb,
		     TOP_packssdw,
		     TOP_packuswb,
		     TOP_pmulhuw,
		     TOP_pavgb,
		     TOP_pavgw,
		     TOP_psadbw,
		     TOP_max64v8,
		     TOP_max64v16,
		     TOP_min64v8,
		     TOP_min64v16,
		     TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, mmx, opnd1);
  Operand(1, mmx, opnd2);

  Instruction_Group("MMX shuffle-int",
		    TOP_pshufw64v16,
		    TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, mmx, opnd1);
  Operand(1, uimm8, opnd3);


  Instruction_Group( "mov int32 2 mmx",
                     TOP_movi32_2m,
                     TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, int32, opnd1);

  Instruction_Group( "mov int64 2 mmx",
                     TOP_movi64_2m,
                     TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, int64, opnd1);

  Instruction_Group( "mov mmx 2 int32",
                     TOP_movm_2i32,
                     TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, mmx, opnd1);

  Instruction_Group( "mov mmx 2 int64",
                     TOP_movm_2i64,
                     TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, mmx, opnd1);

  Instruction_Group( "mov mmx 2 sse",
                     TOP_movq2dq,
                     TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, mmx, opnd1);

  Instruction_Group( "mov sse 2 mmx",
                     TOP_movdq2q,
                     TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, fp64, opnd1);

  Instruction_Group("packed mmx arith",
  		     TOP_psllw_mmx,
  		     TOP_pslld_mmx,
  		     TOP_psrlw_mmx,
  		     TOP_psrld_mmx,
  		     TOP_psraw_mmx,
  		     TOP_psrad_mmx,
  		     TOP_pand_mmx,
  		     TOP_pandn_mmx,
  		     TOP_por_mmx,
  		     TOP_pxor_mmx,
                     TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, mmx, opnd1);
  Operand(1, mmx, opnd2);

  Instruction_Group( "vector cvt sse ps/pd 2 mmx pi",
		     TOP_cvtps2pi,
		     TOP_cvttps2pi,
		     TOP_cvtpd2pi,
		     TOP_cvttpd2pi,
                     TOP_UNDEFINED);
  Result(0, mmx);
  Operand(0, fp128, opnd1);

  Instruction_Group( "vector cvt mmx pi 2 sse ps/pd",
		     TOP_cvtpi2ps,
		     TOP_cvtpi2pd,
                     TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, mmx, opnd1);

  Instruction_Group("int64 load to sse",
		    TOP_ld64_2sse,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, int64, base);
  Operand(1, simm32, offset);

  Instruction_Group("int64 load to sse w/o base or index",
		    TOP_ld64_2sse_n32,
		    TOP_UNDEFINED);
  Result(0, fp64);
  Operand(0, simm32, offset);

  Instruction_Group("int64 store from sse",
		    TOP_store64_fsse,
		    TOP_UNDEFINED);
  Operand(0, fp64, storeval);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int64 store from sse w/o base or index",
		    TOP_store64_fsse_n32,
		    TOP_UNDEFINED);
  Operand(0, fp64, storeval);
  Operand(1, simm32, offset);

  Instruction_Group("fp128 2 int32",
		    TOP_movmskpd,
		    TOP_movmskps,
		    TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, fp128, opnd1);

  Instruction_Group("fp128 ptest int32",
				TOP_ptest,
		    TOP_UNDEFINED);
  Result(0, rflags);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);

  Instruction_Group("fp128 ptest x",
						TOP_ptestx,
						TOP_UNDEFINED);
	Result(0, rflags);
	Operand(0, fp128, opnd1);
	Operand(1, int64, base);
	Operand(2, simm32, offset);

  Instruction_Group("fp128 ptest xx",
						TOP_ptestxx,
						TOP_UNDEFINED);
	Result(0,rflags);
	Operand(0, fp128, opnd1);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);

  Instruction_Group("fp128 ptest xxx",
						TOP_ptestxxx,
						TOP_UNDEFINED);
	Result(0,rflags);
	Operand(0, fp128, opnd1);
	Operand(1, int64, index);
	Operand(2, uimm8, scale);
	Operand(3, simm32, offset);

  Instruction_Group("maskmov mmx",
		    TOP_maskmovq,
		    TOP_UNDEFINED);
  Operand(0, mmx, opnd1);
  Operand(1, mmx, opnd2);

  Instruction_Group("maskmov sse",
		    TOP_maskmovdqu,
		    TOP_UNDEFINED);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);

  Instruction_Group("extract field xmm",
                    TOP_extrq,
                    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);


  Instruction_Group("pmax pmin xmm",
                    TOP_pmaxsb,
                    TOP_pmaxsd,
                    TOP_pmaxuw,
                    TOP_pmaxud,
                    TOP_pminsb,
                    TOP_pminsd,
                    TOP_pminuw,
                    TOP_pminud,
										TOP_phminposuw,
                    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);

  Instruction_Group("insert field xmm",
                    TOP_insertq,
                    TOP_UNDEFINED);
  Result(0, fp128);
  Operand(0, fp128, opnd1);
  Operand(1, fp128, opnd2);

  Instruction_Group("int8 exchange with mem",
                    TOP_xchgx8,
		            TOP_UNDEFINED);
  Result(0, int8);
  Operand(0, int8, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int16 exchange with mem",
                    TOP_xchgx16,
		            TOP_UNDEFINED);
  Result(0, int16);
  Operand(0, int16, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int32 exchange with mem",
                    TOP_xchgx32,
		            TOP_UNDEFINED);
  Result(0, int32);
  Operand(0, int32, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

  Instruction_Group("int64 exchange with mem",
                    TOP_xchgx64,
		            TOP_UNDEFINED);
  Result(0, int64);
  Operand(0, int64, opnd1);
  Operand(1, int64, base);
  Operand(2, simm32, offset);

#include "isa_avx_operands.cxx"
#include "isa_xop_fma_operands.cxx";

  Instruction_Group("tls_get_addr_64",
		    TOP_tls_get_addr_64,
		    TOP_UNDEFINED);
  Result(0, rax);
  Result(1, rdi);
  Operand(0, simm32, offset);

  Instruction_Group("tls_get_addr_32",
		    TOP_tls_get_addr_32,
		    TOP_UNDEFINED);
  Result(0, rax);
  Operand(0, simm32, offset);

  /* CFI */

  Instruction_Group("cfi_one_arg",
		    TOP_cfi_def_cfa_offset,
		    TOP_cfi_def_cfa_register,
		    TOP_UNDEFINED);
  Operand(0, simm32, offset);

  Instruction_Group("cfi_two_arg",
		    TOP_cfi_offset,
		    TOP_UNDEFINED);
  Operand(0, simm32, offset);
  Operand(1, simm32, target);
  
  ISA_Operands_End();
  return 0;
}
